
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;This include file conains the functions:
;copyProtocolSignature This function will make a copy of a ProtocolSignature structure
;copyListenSession / copyListenSessionToClientSession This function will make a copy of a ListenSession structure
;CreateListenSession This function will create and fill in a mem location to a ListenSession structure
;CreateProtocolSession: This function will create and fill in a mem location to a ProtocolSession structure
;copyControlPanelOption: This function will return a copy of the passed pointer to a ControlPanelOption structure in a linked list form (add extra 4/8 bytes for a pointer on top)
;copyValueByType: will create a copy of a value by their type (to be placed in a UserValueOption structure)
;copyUserValueOption: This function will make a copy of the UserValueOption
;copyMultiColumnList: This function will make a copy of a MultiColumnList structure
;copyRows: This function will add and validate rows to a list
;copyInternalRowsCollection: This function will make a copy of a internal Row collection 1:rowCollection

;------------------------------------------------------------------------------
;This function will make a copy of a ProtocolSignature structure
;input:
;  1:pointer to ProtocolSignature Structure
;output: EAX: Pointer to copy of ProtocolSignature Structure
;------------------------------------------------------------------------------
copyProtocolSignature:
call denter

mov pesi,pdword [pebp+psize*2]
test pesi,pesi
cmovz peax,pesi
jz endCopyProtocolSignature

xor ecx,ecx

countProtocolSignatureSize:
  lodsd ;Load the size member of the item
  add pesi,peax ;Go to next ProtocolSignature structure

  ;add size to count
  add ecx,eax

  jecxz endCopyProtocolSignature   ;If ECX is zero, there are no ProtocolSignature structures

  ;include the 4 bytes of the integer
  add ecx,4
  test eax,eax ;if the size of the buf member is zero, this is the last ProtocolSignature structure
jnz countProtocolSignatureSize

sub pesi,pecx ;set ESI back to the first ProtocolSignature structure

;allocate enough size to copy the ProtocolSignature structures
push pecx
call GetMemLocation
mov pedi,peax

mov edx,ecx ;save the size of the structure

rep movsb ;Copy the ProtocolSignature structures

sub pedi,pedx ;set EDI back to the first ProtocolSignature structure
mov peax,pedi ;set return value

endCopyProtocolSignature:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will make a copy of a ListenSession structure
;input:
;  1:pointer to ListenSession Structure
;output: EAX: Pointer to copy of ListenSession Structure
;------------------------------------------------------------------------------
copyListenSessionToClientSession:
copyListenSession:
call denter

;pointer to ListenSession on (R/E)BX and data param on (R/E)SI and a copy on (R/E)DI
mov pebx,pdword [pebp+psize*2]
mov pesi,pdword [pebx+8]
mov pedi,pesi
xor ecx,ecx ;count
xor edx,edx ;new data param

test pesi,pesi
jz noDataParam

nextProtocolDataStruct:
  lodsd ;Get size param
  test eax,eax
  jz lastDataParam

  add pecx,peax ;Add size to count
  add pesi,peax ;add size to pointer to get next structure
  sub pesi,4    ;the size of the size parameter is already increment by lodsd, substract it
jmp nextProtocolDataStruct

lastDataParam:
add pecx,4

push pecx
call GetMemLocation
mov pesi,pedi ;original data param
mov pedi,peax ;the copy of the data param
mov pedx,peax ;the copy of the data param

;Divide ECX by 4 and add the rest to round the value on 4
;This so we can preform a 4 byte copy per cycle (movsd) instead of 1 byte per cycle (movsb), it's faster
;--;shl pecx,1 ;ECX /=  2
;--adc pecx,0 ;ECX += carry
;--shl pecx,1 ;ECX /= 2
;--adc pecx,0 ;ECX += carry

rep movsb;d ;copy the data strucutre, 4 bytes a cycle

noDataParam:
push pedx
call CreateListenSession

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will create and fill in a mem location to a ListenSession structure
;input:
;  1:pointer to Data parameter
;output: EAX: Pointer to ListenSession Structure
;------------------------------------------------------------------------------
CreateListenSession:
call denter

push SIZEOF_LISTENSESSION
call GetMemLocation
mov pebx,peax

;fill in the data meber
mov peax,pdword [pebp+psize*2]
mov pdword [pebx+8],peax

;fill in the size member
mov pdword [pebx],SIZEOF_LISTENSESSION

mov peax,pebx

call dleave
ret (psize*1)

;------------------------------------------------------------------------------
;This function will create and fill in a mem location to a ProtocolSession structure
;input:
;  1:Protocolname
;output: EAX: Pointer to ProtocolSession
;------------------------------------------------------------------------------
CreateProtocolSession:
call denter

push SIZEOF_PROTOCOLSESSION
call GetMemLocation
mov pebx,peax

;copy protocolName to new pointer and fill in the menmber protocolName
push pdword [pebp+(psize*2)]
call copyStringA
mov pdword [pebx+(psize*1)],peax

;fill in the size member
mov pdword [pebx],SIZEOF_PROTOCOLSESSION

mov peax,pebx

call dleave
ret (psize*1)

;------------------------------------------------------------------------------
;This function will return a copy of the passed pointer to a ControlPanelOption structure in a linked list form (add extra 4/8 bytes for a pointer on top)
; 1:ControlPanelOption
; 2:flags (0x1: use copyInternalRowsCollection instead of copyRows)
;------------------------------------------------------------------------------
copyControlPanelOptiona:
call denter
MakeStack

mov peax,pdword [pebp + (psize*2)] ;return value
test peax,peax
jz _endcopyControlPanelOption

;make a pointer to a ControlPanelOption
push SIZEOF_CONTROLPANELOPTION+psize
call GetMemLocation

if used is64
   mov r12,rax
   mov rsi,qword [rbp+(psize*2)] ;pointer to ControlPanelOption
   mov rdi,rax
else
   mov edi,eax
   xchg dword [pebp+0x8],eax
   mov esi,eax
end if

;mov pesi,pedi
add pedi,4+psize

;get size of structure
;----mov pesi,pdword [pebp+(psize*2)] ;pointer to ControlPanelOption

lodsd   ;Comparand
xor edx,edx
xchg eax,edx
mov ecx,SIZEOF_CONTROLPANELOPTION
cmpxchg edx, ecx    ;Destination, Exchange,
;edx = size

sub edx,4 ;subtract the member size

;return if zero
test edx,edx
jz endcopyControlPanelOption

;copy flags
lodsd
stosd
mov ebx,eax

;substract read size, return if zero
sub edx,4
test edx,edx
jz endcopyControlPanelOption

mov ecx,7

;copy the pointer (strings) and it's content
copyMembersLoop:
        lodsp ;macro: 64:lodsq 32:lodsd

        ;is this member name
        cmp ecx,7
        jne isNotParamName

        ;check if flag GDPD_NAME_IS_INTEGER is set
        test ebx,GDPD_NAME_IS_INTEGER
        jz isNotFlagNameIsInteger

        ;store the integer and do the loop
        stosp
        jmp nextCopyMembersLoop

        isNotFlagNameIsInteger:

        ;--
        isNotParamName:
        test peax,peax
        jz errorCopyMembersLoop

        ;check pointer of member
        push errorCopyMembersLoop
        push RIGHT_READ
        push peax
        call saveCheckPointer

        ;create new pointer using copyStringA or copyStringW (called below)
        push peax
        mov peax,copyStringA

        ;is this member optionName
        cmp ecx,6
        jne isNotParamOptionName

        ;check if flag CPO_OPTIONNAME_IS_UNICODE is set
        test ebx,CPO_OPTIONNAME_IS_UNICODE
        jz isNotParamOptionName

        mov peax,copyStringW ;unicode

        isNotParamOptionName:
        ;is this member description
        cmp ecx,7
        jne isNotParamDescription

        ;check if flag CPO_DESCRIPTION_IS_UNICODE is set
        test ebx,CPO_DESCRIPTION_IS_UNICODE
        jz isNotParamDescription

        mov peax,copyStringW ;unicode

        isNotParamDescription:

        call peax ;copyStringA or copyStringW
        jmp writePointer


errorCopyMembersLoop:
        xor eax,eax

writePointer:
        stosp

nextCopyMembersLoop:
        sub edx,psize
        test edx,edx
        jz endcopyControlPanelOption
dec ecx
test ecx,ecx
jnz copyMembersLoop

lodsp
mov pesi,peax

;check pointer of member userValue
push endcopyControlPanelOption
push RIGHT_READ
push peax
call saveCheckPointer

;check and copy defaultValue and jump to endcopyControlPanelOption if an error accurd
push pdword [pebp+psize*3];flags
push 0 ;we desire a new pointe to the structure
push endcopyControlPanelOption
push pesi
call copyUserValueOptiona

stosp

endCopyMembersLoop:


endcopyControlPanelOption:
if used is64
   mov rax,r12
else
   mov eax,dword [pebp+0x8]
end if

_endcopyControlPanelOption:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;This function will make a copy of the Value by type 1:type, 2:value to copy 3:&size 4:errReturnFunc 5:flags (0x1: use copyInternalRowsCollection instead of copyRows)
;------------------------------------------------------------------------------
copyValueByTypeab:
call denter

mov ecx,dword [pebp+psize*2] ;the type
mov pesi,pdword [pebp+psize*4] ;size
test pesi,pesi
je noSizePassed

mov esi,dword [pesi]


noSizePassed:
;ASCII String
jecxz copyPointer

;Unicode string
dec ecx
jz copyPointer

if used is64
   ;boolean, char, short int, integer, large integer, date, time and dateTime
   sub ecx,9                ;cmp ecx,8
   jb copyDwordOrQword      ;jbe copyDwordOrQword
                            ;sub ecx,9

    ;progress
   jz copyProgress
else
   ;boolean, char, short int, integer
   sub ecx,5                ;cmp ecx,4
   jb copyDwordOrQword      ;jbe copyDwordOrQword
                            ;sub ecx,5

   ;large integer
   jz copyPointer

   ;date and time
   sub ecx,3                ;cmp ecx,2
   jb copyDwordOrQword      ;jbe copyDwordOrQword
                            ;sub ecx,3

   ;dateTime
   jz copyPointer

   ;progress
   dec ecx
   jz copyProgress
end if

;list
dec ecx
jz copyList

mov eax,UVO_WRONG_TYPE
jmp errCopyProgressOption

;--
copyProgress:
mov eax,UVO_WRONG_TYPE
;check if it is a zero pointer, progress that is zero is invalid, error
mov pecx,pdword [pebp+psize*3] ;value
test pecx,pecx
jz errCopyProgressOption

test esi,esi
jz copyCurrentProcessPointer

mov eax,CORRUPT_SIZE
;fill in the size of the strucute
cmp esi,SIZEOF_CURRENTPROGRESS
jb errCopyProgressOption

copyCurrentProcessPointer:
mov esi,SIZEOF_CURRENTPROGRESS

;--
copyPointer:
;continue if there is no defaultValue
mov peax,pdword [pebp+psize*3]
test peax,peax
je endCopyProgressOption

;check pointer of member userValue
push errCopyProgressOption
push RIGHT_READ
push peax
call saveCheckPointer

test esi,esi
jnz copyMemoryBySize

mov ecx,dword [pebp+psize*2] ;the type
jecxz countAsciiString

push pdword [pebp+psize*3]
call strlenW
add eax,2 ;zero byte
jmp copyMemoryBySizeStrlen

countAsciiString:
push pdword [pebp+psize*3]
call strlen
inc eax ;zero byte

copyMemoryBySizeStrlen:
mov esi,eax

copyMemoryBySize:
;create new memory location and fill in the member defaultValue
push pesi
call GetMemLocation
mov pedi,peax

push pesi               ;defaultValueSize
push pdword [pebp+psize*3] ;defaultValue
push peax
call strcpyn
jmp endCopyProgressOption

;--
copyDwordOrQword:
mov pedi,pdword [pebp+psize*3] ;get defaultValue
xor esi,esi                    ;clear size since returnvalue will not be a pointer
jmp endCopyProgressOption

;--
copyList:
;check pointer of member userValue
push errCopyProgressOption
push RIGHT_READ
push pdword [pebp+psize*3]
call saveCheckPointer

;get size param
mov pesi,pdword [pebp+psize*3]
mov esi,dword [pesi]

;make a copy of the MultiColumnList
push pdword [pebp+psize*6] ;flags
push errCopyProgressOption
push pdword [pebp+psize*3]
call copyMultiColumnLista
mov pedi,peax

;--return
endCopyProgressOption:
;return the size of the item
mov peax,pdword [pebp+psize*4] ;&size
test peax,peax
je noSizePassed2

mov dword [peax],esi

noSizePassed2:
;return the value itself
mov peax,pedi
call dleave
ret (psize*5)

;--Error
errCopyProgressOption:
;return to the error pointer
call dleave
add pesp,psize*6
jmp pdword [pesp-psize*2]


;------------------------------------------------------------------------------
;This function will make a copy of the UserValueOption
;1:pointer to UserValueOption
;2:pointer to jump to if something go's wrong
;3:pointer to output UserValueOption (can be zero)
;4:flags (0x1: use copyInternalRowsCollection instead of copyRows)
;------------------------------------------------------------------------------
copyUserValueOptiona:
call denter

;get the pointer ofthe dest UserValueOption structure from the stack
mov pedi,pdword [pebp+(psize*4)]
test pedi,pedi
jnz useTheirPointer

;no pointer is passed, create our own
push SIZEOF_USERVALUEOPTION
call GetMemLocation
mov pedi,peax

useTheirPointer:
;get type from UserValueOption
mov pesi,pdword [pebp+(psize*2)]
mov ecx,dword [pesi]

;fill in the type member on dest strcuture
mov dword [pedi],ecx

mov pebx,pedi ;set new UserValueOption on ebx/rbx
add pedi,4 ;set edi/rdi on defaultValueSize member


;fill in the size on the pointer
mov edx,dword [pesi+0x4]
mov dword [pedi],edx


push pdword [pebp+psize*5] ;flags
push errCopyDefaultUserValueOption
push pedi ;&size
push pdword [pesi+0x8] ;value
push pecx  ;type
call copyValueByTypeab
mov pdword [pedi+4],peax ;fill in the defaultValue

;--return
endCopyDefaultUserValueOption:
mov peax,pebx
call dleave
ret (psize*4)

;--Error
errCopyDefaultUserValueOption:
;release the pointer if we created it
cmp pdword [pebp+(psize*4)] ,0
jne dontReleasePointerCopyDefaultUserValueOption

push peax

push pebx
call FreeMemLocation

pop peax

dontReleasePointerCopyDefaultUserValueOption:
;return to the error pointer
call dleave
add pesp,psize*5
jmp pdword [pesp-psize*3]

;------------------------------------------------------------------------------
;This function will make a copy of a MultiColumnList structure
;  1:MultiColumnList
;  2:returnpointer on error
;  3:flags (0x1: use copyInternalRowsCollection instead of copyRows)
;------------------------------------------------------------------------------
copyMultiColumnLista:
call denter

mov pesi,pdword [pebp+(psize*2)] ;get pointer to MultiColumnList to copy

;get size member
mov ecx,dword [pesi]

mov eax,CORRUPT_SIZE

cmp ecx,SIZEOF_MULTICOLUNMLIST
jb  errCopyMultiColumnList

push pecx
call GetMemLocation
mov pebx,peax
mov pedi,peax

;copy member size and flags
if used is64
   lodsq
   stosq
else
   lodsd
   stosd
   lodsd
   stosd
end if

sub ecx,8 ;substract the size of member 'size' and 'flags'
;--cmp ecx,SIZEOF_COLUMN  one column is obligated
;--jb doCopyRows

;local edx and edi\rdi
copyUserValueOptions:
  lodsp ;get pointer to optionName

  test peax,peax
  jz noOptionName

  mov pedx,peax ;(R/e)BX => optionName

  ;calculate the size of the optionName string
  push pedx
  call strlen
  inc eax ;include space for a zero terminator

  push peax
  call GetMemLocation

  push pedx ;already pushed
  push peax

  mov pedx,peax ;(R/E)DX => copy of optionName

  call strcpy

  mov peax,pedx

  noOptionName:
  stosp ;store copy of optionName

  ;check if the type is an existing one
  lodsd
  mov edx,eax
  mov eax,UVO_WRONG_TYPE

  cmp edx,10
  ja errCopyMultiColumnList

  mov eax,edx

  stosd ;store type

  push pdword [pebp+psize*4]  ;push flags
  push errCopyMultiColumnList ;error return pointer
  push pesi                   ;pointer to size of DefaultValue

  lodsd ;load size
  stosd ;store size

  lodsp ;get defaultValue

  ;-push pebx (already done) ;&size of value
  push peax ;defaultValue
  push pedx ;type
  call copyValueByTypeab

  stosp ;store copies defaultValue

  ;-push peax ;push size for use strcpyn

  ;-push peax
  ;-call GetMemLocation

  ;-stosp ;Save the new pointer of defaultValue
  ;-mov pedx,peax

  ;-lodsp ;load pointer to defaultValue on eax/rax

  ;-;check pointer of member
  ;-push errCopyMultiColumnListAndPop
  ;-push RIGHT_READ
  ;-push peax
  ;-call saveCheckPointer

  ;-;push size already done up here
  ;-push peax ;src
  ;-push pedx ;des
  ;-call strcpyn

  sub ecx,SIZEOF_COLUMN ;substract the size of a COLUMN strcuture
  cmp ecx,SIZEOF_COLUMN
jae copyUserValueOptions

doCopyRows:
cmp ecx,psize
jb endCopyMultiColumnList

lodsp ;load pointer to the first row

test dword [pebp+psize*4],0x1 ;FLAG_USE_COPYINTERNALROWSCOLLECTION
jz justCopyRows

  push peax ;rows
  call copyInternalRowsCollection
  jmp gotCopyRows

justCopyRows:
  push pdword [pebp+psize*4] ;flags
  push 0    ;no collecion does already exists
  push peax ;rows
  push pdword [pebp+(psize*2)] ;MultiColumnList
  call copyRowsa

gotCopyRows:
stosp


;return
endCopyMultiColumnList:
mov peax,pebx

;add peax,8+4*2+psize*2
call dleave
ret psize*3

;--Error
;--errCopyMultiColumnListAndPop:
;--add pesp,psize

;--push peax

;--push pedx
;--call FreeMemLocation

;--pop peax

errCopyMultiColumnList:
;release memory
push peax

push pebx
call FreeMemLocation

pop peax

;return to the error pointer
call dleave
add pesp,psize*4
jmp pdword [pesp-psize*2]

;------------------------------------------------------------------------------
;This function will add and validate rows to a list 1:MultiColumnList, 2:addRowList, 3:UseCollectionForNewRows 4:flags (0x1: use copyInternalRowsCollection instead of copyRows)
;------------------------------------------------------------------------------
copyRowsa:
call denter

mov pesi,pdword [pebp+psize*2] ;MultiColumnList
if used is64
  lodsd ;get size member MultiColumnList
  add pesi,4 ;skip size and flags
  sub eax,SIZEOF_MULTICOLUNMLIST
else
  mov ecx,dword [pesi] ;get size member MultiColumnList
  add pesi,8 ;skip size and flags
  sub ecx,SIZEOF_MULTICOLUNMLIST
end if

;Do a divide by the size of COLUMN strcuture to get the ammount of COLUMN structures
;multiply by the size of a integer to get the size needed for a structure with the types
if used is64
  ;divide by 24 (the size of the structure)
  xor edx,edx
  mov ecx,24
  div ecx

  mov ecx,eax ;ecx= ammount of UserValueOption structures

  shl ecx,2 ;ecx=ecx*4 (the size of a dword)
else
  shr ecx,2 ;divide by 16 (the size of structure) AND multiply by 4 (a dword)
end if

push pecx
call GetMemLocation
mov pedi,peax ;pointer of temp structure with types of every item in a row

mov edx,ecx ;backup the size of the structure with types

;get the ammount of items in 1 row
if used is64
  shr ecx,2 ;divide by 4
else
  shr ecx,2 ;divide by 4
end if

;Fill in the types in the structure
fillInStructureWithType:
  lodsp ;skip defaultPointer

  ;load type from rsi/esi and store in rdi/edi
  lodsd
  stosd

  ;skip the defaultValueSize and defaultValue member,
  ;  add pesi,psize+4, but this is smaller
  ;can replace this, but using lods is smaller
  lodsd
  lodsp
loopd fillInStructureWithType

;EDX = the size of the structure with types
;EDI = pointer to structure with the types

mov ecx,edx ;backup the size of the structure with types
;get the ammount of items in 1 row
if used is64
  shr ecx,2 ;divide by 4
else
  shr ecx,2 ;divide by 4
end if

sub pedi,pedx
inc edx ;add one empty terminating pointer

if used is64
  shl edx,2 ;multiply by 4 (to get the array for pointers and size)
else
  shl edx,1 ;multiply by 2 (to get the array for pointers and size)
end if

mov pebx,pdword [pebp+psize*4]   ;UseCollectionForNewRows
test pebx,pebx
jnz useExistingRowList

  call CreateCollection
  mov pebx,peax ;EBX = list with Rows

  ;add pointer to MultiColumnList
  push pdword [pebp+psize*2]  ;MultiColumnList
  push pebx
  call AddDword

  ;add ammount of items per row
  push pecx
  push pebx
  call AddDword


useExistingRowList:
;xor pebx,pebx ;clear register for new Row
mov pesi,pdword [pebp+psize*3] ;rows

test pesi,pesi
jz endCopyRowsAndFreeList

if used is64
  xor r12,r12 ;lastRow
else
  push 0 ;lastRow
end if


;-r12 or dword [esp] = zero (lastRow)
;-(R/E)SI = pointer to the Rows structures to add
;-(R/E)EBX = list with rows
;-(R/E)DX = the size of the structure with enough space to hold the pointers to the values in the row
;-(R/E)DI = pointer to structure with the types

loopRows:
  ;read the size
  lodsd
  sub eax,8 + psize ;substract member size, id and nextrow
  mov ecx,eax
  cmp ecx,edx ;compair if all values are avaibable
  jbe sizeAcceptable

  mov ecx,edx

  sizeAcceptable:
  ;create new Row and add it to the list
  push pedx
  call GetMemLocation

  if used is64
    mov r12,rax ;lastRow
  else
    push eax ;lastRow
  end if

  ;-;fill in the id and higher it
  ;-mov dword [peax+0x4],ebx
  ;-inc ebx

  ;add the row to the list
  push peax
  push pebx
  call AddDword

  if used is64
    shr ecx,3 ;divide by 8
  else
    shr ecx,2 ;divide by 4
  end if

  lodsd ;skip the id member
  lodsp ;skip nextRow member

  ;64=save current pos on r13, 32=save on stack
  if used is64
    ;-----mov qword [r13],rax ;item
    mov r13,r12
    ;--add r13,8+psize
    ;save the pointer to the list with types
    mov r14,rdi
    mov r15,rsi
  else
    ;--;get the ROW structure pointer and push it on stack again
    pop eax
    push eax
    ;save pointer to list with types
    push edi
    ;save pointer to row
    push esi
    ;push the pointer to member items on stack
    push eax

  end if

  jecxz noItemsToCopyToNewRow

  loopItemsInRow:
    lodsp

    push 0

    push pdword [pebp+psize*5] ;flags
    push errorCopyRows
    push pesp ;&pointer current stack pos
    add pdword [pesp],psize*2 ;set pushed stack on &size
    push peax ;value
    mov eax,dword [pedi]
    push peax  ;type
    push noErrorCopyRows
    jmp copyValueByTypeab

    errorCopyRows:
    xor peax,peax
    mov pdword [pesp],peax ;zero type

    noErrorCopyRows:


    ;save pointer of value on the strcuture
    if used is64
      pop qword [r13]     ;size
      add r13,8
      mov qword [r13],rax ;item
      add r13,8
    else
      xchg eax,dword [pesp+4] ;the buffer with enough space for size and itempointer
      pop dword [peax] ;size
      add eax,4
      pop dword [peax] ;item
      add eax,4
      push eax
    end if

    ;get next type on list
    add pedi,4
  loopd loopItemsInRow

  noItemsToCopyToNewRow:
  if used is64
    mov rdi,r14 ;restore pointer to list with types
    mov rsi,r15
  else
    add pesp,0x4
    pop esi
    pop edi  ;restore pointer to list with types
    ;add pesp,0x4
  end if

  ;get next row and test on zero
  mov pesi,pdword [pesi-psize]
  test pesi,pesi
jnz loopRows

endCopyRowsAndFreeList:
push pedi
call FreeMemLocation

;--
endCopyRows:
mov peax,pebx ;list with rows

call dleave
ret psize*4



;------------------------------------------------------------------------------
;This function will make a copy of a internal Row collection 1:rowCollection
;------------------------------------------------------------------------------
copyInternalRowsCollection:
call denter

;copy the list (with original pointers
push pdword [pebp+psize*2]
call copyDwordList
mov pebx,peax

;get ammount of items per row
push 2
push pebx
call GetDwordItem

if used is64
  shl eax,4 ;multiply by 16 (to get the array for pointers and size)
else
  shl eax,3 ;multiply by 8 (to get the array for pointers and size)
end if

mov edx,eax ;size of one row

push pebx
call CountItems
sub eax,2   ;substract the pointer to MultiColumnList and items per row
mov ecx,eax ;ECX = ammount of rows
jecxz noRowsToCopy

;EBX = copy of the list
;EDX = size of one row
;ECX = ammount of rows
copyInternalRows:
  mov edi,ecx ;EDI = current item
  add edi,2   ;incemrent the pointer to MultiColumnList and items per row

  ;get row
  push pedi
  push pebx
  call GetDwordItem

  push pedx ;size of a row
  push peax ;row to copy
  call copyBuffer
  mov pesi,peax ;ESI = row

  ;set copy of row to the list
  push peax ;copy of the row
  push pedi ;itemId
  push pebx ;collection
  call ModifyDword

  mov pedi,pesi ;EDI = row

  if used is64
    mov r12,rdx ;R12 = size of one row
    shr r12,4   ;divide by 16 (to get the array for pointers and size)
  else
    push edx   ;save size of one row
    shr edx,3  ;divide by 8 (to get the array for pointers and size)
  end if

  copyInternalItems:
    lodsp     ;get the size of the item
    push peax ;push the size of the item
    stosp     ;store the size of the item

    lodsp     ;get pointer to the item
    push peax ;push the pointer to the item

    ;copy the buffer of the item
    call copyBuffer

    stosp     ;store the copy of the item

    if used is32
      dec edx
    else
      dec r12
    end if
  jnz copyInternalItems ;copy next item

  if used is32
    pop pedx ;resotre size of one row
  end if

loop copyInternalRows

noRowsToCopy:
mov peax,pebx

call dleave
ret psize*1