
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will handle the connection
;------------------------------------------------------------------------------
HandleConnection:
StartProc 2 ;Set arguments on stack
call denter

xor eax,eax ;return value

mov pedx,pdword [pebp+psize*2]  ;psess

cmp [psess_http],pedx
jne endHandleConnection
;
;Handle HTTP protocol
;

if used is32
  sub pesp,8*5 ;make room for 5 long long's
end if

;create size for a ResponseHeader structure
sub pesp,sizeOfResponseHeaderSize

clientLoop:
  ;zero structure and long longs
  mov pedi,pesp
  if used is32
    mov ecx,sizeOfResponseHeaderSize+(8*5)
  else
    mov ecx,sizeOfResponseHeaderSize
  end if
  xor eax,eax
  rep stosb

  mov esi,0x1 ;GET_FROM_CLIENTSESSION
  mov eax,CPO_NAME_MAX_SIZE
  call GetPluginDataInt
  mov pedx,pecx ;(R/E)DX => maximum allowed size

  ;mov esi,0x1 ;GET_FROM_CLIENTSESSION
  mov eax,CPO_NAME_MAX_TIMEOUT
  call GetPluginDataInt ;(R/E)CX => maximum allowed timeout

  if used is32
    ;set contentLength
    lea eax,dword [pesp+sizeOfResponseHeaderSize]
    mov dword [pesp+8+(psize*3)],eax
    ;set contentRangeOffset
    lea eax,dword [pesp+8*1+sizeOfResponseHeaderSize]
    mov dword [pesp+8+(psize*4)],eax
    ;set contentRangeTotalLength
    lea eax,dword [pesp+8*2+sizeOfResponseHeaderSize]
    mov dword [pesp+8+(psize*5)],eax
    ;set lastModified
    lea peax,dword [pesp+8*3+sizeOfResponseHeaderSize]
    mov dword [pesp+8+(psize*6)],eax
    ;set expires
    lea peax,dword [pesp+8*4+sizeOfResponseHeaderSize]
    mov dword [pesp+8+(psize*7)],eax

    ;add dword [esp+8+(psize*3)],sizeOfResponseHeaderSize
  end if

  ;set members from the ResponseHeader structure
  mov dword [pesp+4*2+psize],TEXT_HTML ;contentType
  mov pdword [pesp+8],200 ;set status code

  push 0

  ;-------------
  ;Receive HTTP header
  ;-------------
  push pesp ;*error
  push RPF_USE_TIME_LIMIT or RPF_USE_HEADER_SIZE_LIMIT or RPF_CHECK_VALID_METHOD ;flags
  push pecx         ;maxTimeToReceive
  push pedx         ;maxHeaderSizeLimit
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [RecvAndParseHTTP]
  mov pebx,peax

pop peax ;error code

test pebx,pebx
jnz noErrorRecvHTTPHeader

  dec eax
  jz send413Error ;RPE_EXCEEDED_SIZE_LIMIT
  jmp endHandleConnection

  send413Error:
    ;set content length
    if used is32
      mov dword [pesp+4*2+psize*3],sizeContent413Error
    else
      mov qword [rsp+4*2+psize*3],endContent413Error-content413Error
    end if

    mov dword [pesp+8],413 ;set statusCode

    mov edi,endContent413Error-content413Error ;set size of content to send
    mov pesi,content413Error                   ;pointer to content to send
    jmp sendError

  send404Error:
    ;set content length
    if used is32
      mov dword [pesp+4*2+psize*3],sizeContent404Error
    else
      mov qword [rsp+4*2+psize*3],endContent404Error-content404Error
    end if

    mov dword [pesp+8],404 ;set statusCode
    ;-or dword [pesp],FRH_CONNECTION_KEEP_ALIVE

    mov edi,endContent404Error-content404Error ;set size of content to send
    mov pesi,content404Error                   ;pointer to content to send
    jmp sendError

  sendWrongURLError: ;redirect this error to '/'
    mov dword [pesp+8],302 ;set statusCode
    mov pdword [pesp+8+(psize*9)+(psize*6)],baseLocation ;member location = '/'
    or dword [pesp],FRH_FORCE_ZERO_CONTENT_LENGTH ;add flag

    ;set content length
    if used is32
      mov dword [pesp+4*2+psize*3],zeroLength
    end if

    xor edi,edi ;zero size of content to send
    jmp sendError

  sendIsDirectoryWithoutSlashError: ;redirect this to URL + '/'
    mov pedi,pdword [pebx+psize*5] ;URLbuffer

    ;mov pdword [pesp+4*2+9*psize+4*2+5*psize],pedi ;member location = URL+'/'
    mov pdword [pesp+8+(psize*9)+(psize*6)],pedi ;member location = URL+'/'

    ;(R/E)CX => zero
    xor ecx,ecx
    not pecx  ;(R/E)CX = -1
    mov al,0
    repnz scasb

    mov word [pedi-1],'/' ;add slash add the end of the string

    mov dword [pesp+8],302 ;set statusCode

    or dword [pesp],FRH_FORCE_ZERO_CONTENT_LENGTH ;add flag

    ;set content length
    if used is32
      mov dword [pesp+4*2+psize*3],zeroLength
    end if

    xor edi,edi ;zero size of content to send
    jmp sendError

  sendError:
    ;send HTTP-header
    mov peax,pesp

    push DEFAULT_SENDTIMEOUT   ;timeout
    push peax                  ;ResponseHeader
    push pdword [pebp+psize*3] ;ClientSession
    push pdword [pebp+psize*2] ;ProtocolSession
    call [SendHTTPHeader]


    test pedi,pedi
    jz dontSendContent

    mov ecx,edi   ;size of buf
    call shortSendData

    dontSendContent:
    jmp closeHandleConnection


noErrorRecvHTTPHeader:
  mov pecx,pdword [pebx] ;DefaultHTTPVariables
  mov peax,pdword [pecx+13*psize] ;connection

  ;generate a hash of the value connection
  push peax ;name of the variable
  call GenerateHashCaseInsensitiveSafe
  cmp eax,0xC2798F5C ;hash of string 'Keep-Alive'
  jne isVersion1 ;don't set FRH_CONNECTION_KEEP_ALIVE flag if the client didn't requested to keep the connection alive

  mov peax,pdword [pecx] ;pointer to httpVersion
  mov eax,dword [peax]    ;get foure bytes from the version

  cmp eax,'1.0'
  je isVersion1 ;set header-field in response header as 'close', since 'Keep-alive' is not allowed in 1.0.

    mov esi,0x1 ;GET_FROM_CLIENTSESSION
    mov eax,CPO_NAME_KEEPALIVE
    call GetPluginDataInt ;(R/E)CX => boolean KEEPALIVE
    test ecx,ecx
    jz isVersion1

      or dword [pesp],FRH_CONNECTION_KEEP_ALIVE

  isVersion1:
  mov pedi,pdword [pebx+psize*5] ;URLbuffer

  push 0
  push pedi
  call [ConvertEscapedCharactersToBytes]

  push pedi
  call [DeleteDoubleSlashesInUrl]

  push pedi
  call [DecodeURL]
  test eax,eax
  jz sendWrongURLError ;the url has to many parent directories / hack atempt

  ;get redirection if set
  mov eax,CPO_NAME_REDIRECTIONS
  call SearchForDomainAndSpecificItem
  jz noRedirections

  sendRedirect:
    ;start redirecting

    ;get redirect path
    push COLUMN_REDIR_TO ;columnId
    push pesi            ;rowId
    push pecx            ;;pointer to list
    call GetListItemPointer
    mov pecx,peax ;(R/E)CX => pointer to redirect location

    mov pdword [pesp+8+(psize*1)],TEXT_HTML ;set text/html if the extension has no mime-type
    mov dword [pesp+8],302 ;set statusCode
    mov pdword [pesp+8+(psize*9)+(psize*6)],peax ;set location

    ;send HTTP-header
    mov peax,pesp

    push DEFAULT_SENDTIMEOUT   ;timeout
    push peax                  ;ResponseHeader
    push pdword [pebp+psize*3] ;ClientSession
    push pdword [pebp+psize*2] ;ProtocolSession
    call [SendHTTPHeader]

    ;free the location
    push pecx
    call FreeMemLocation
    jmp closeHandleConnection

noRedirections:

  ;get WWWroot
  push 0
  push 0x1 ;GET_FROM_CLIENTSESSION
  push CPO_NAME_PATH ;name
  call GetPluginDataPointerSave
  mov pesi,peax ;(R/E)SI => base url

  ;create new URL
  push RELEASE_STRING_1  ;flags
  push pedi  ;requested url
  push pesi  ;base path
  call combineStrings
  mov pedx,peax ;(R/E)DX => Path to requested file

  push pedx
  call isDirectory
  jz UrlIsNotDirectory

    ;request directory listening settings and see if we can send the content of the directory
    mov esi,0x1 ;GET_FROM_CLIENTSESSION
    mov eax,CPO_NAME_DIRLISTEN
    call GetPluginDataInt ;(R/E)CX => boolean Directory listening
    test ecx,ecx
    jz UrlIsNotDirectory

    push pedx
    call strlen

    cmp byte [peax+pedx-1],'\'
    je DirectoryUrlIsValid

    cmp byte [peax+pedx-1],'/'
    jne sendIsDirectoryWithoutSlashError

    mov pdword [pesp+8+(psize*1)],TEXT_HTML ;set text/html if the extension has no mime-type
    mov dword [pesp+8],200 ;set statusCode

    if used EnableChunking
      or dword [pesp],FRH_DONT_END_HEADER or FRH_TRANSFER_ENCODING_CHUNKED ;add flags
    else
      test dword [pesp],FRH_CONNECTION_KEEP_ALIVE
      jz keep_alive_flag_is_not_set

        xor dword [pesp],FRH_CONNECTION_KEEP_ALIVE ;delete flag, because HTTP/1.0 does not suppport Keep-Alive connections

      keep_alive_flag_is_not_set:
    end if

    ;send HTTP-header
    mov peax,pesp

    push DEFAULT_SENDTIMEOUT   ;timeout
    push peax                  ;ResponseHeader
    push pdword [pebp+psize*3] ;ClientSession
    push pdword [pebp+psize*2] ;ProtocolSession
    call [SendHTTPHeader]

    push pedi                  ;URL
    push pedx                  ;Path
    push pdword [pebp+psize*3] ;sess
    push pdword [pebp+psize*2] ;psess
    call SendDirectoryContent

    ;free the base path + URL
    push pedx
    call FreeMemLocation
    jmp closeHandleConnection

  DirectoryUrlIsValid:

UrlIsNotDirectory:
  ;get CGI-application

  ;calculate the size of the request URL
  push pedi
  call strlen
  add pedi,peax  ;Get to the end of the request URL
  mov pecx,peax  ;ECX => size of the string

  ;Start seacrhing for the extension
  std ;Set search direction backwards
  mov al,'.' ;search for a dot
  repnz scasb ;start searching
  cld ;Set search direction forward
  jnz noCGIApplication ;No dot found

  add pedi,2

  mov eax,CPO_NAME_CGI
  call SearchForDomainAndSpecificItem
  jz noCGIApplication

  sendCGI:
    ;check if script exists
    push 0x0 ;READ
    push pedx
    call openFileHandle

    push peax
    call closefile

    inc peax
    test peax,peax
    jz noCGIApplication ;jump if script does not exist

    ;get redirect path
    push COLUMN_CGI_APP ;columnId
    push pesi            ;rowId
    push pecx            ;;pointer to list
    call GetListItemPointer
    mov pecx,peax ;(R/E)CX => pointer to CGI location

    push pecx ;save (R/E)CX

    lea pecx,pdword [pesp+psize] ;(R/E)CX => pointer to ResponseHeader

    push 0

    push pedx
    push 0x1 or 0x40000
    call pushPathTranslated
    db 'PATH_TRANSLATED=',0
    pushPathTranslated:
    push 0x1

    ;get contentType from HTTPSession structure
    mov pesi,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov pesi,pdword [pesi+psize*14] ;get contentType

    test pesi,pesi
    jz noContentType

      push pesi
      push 0x1 or 0x40000
      call pushContentTypeString
      db 'CONTENT_TYPE=',0
      pushContentTypeString:
      push 0x1

    noContentType:

    ;get contentLength from HTTPSession structure
    mov pesi,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov pesi,pdword [pesi+psize*18] ;get contentLength

    test pesi,pesi
    jz noContentLength

      push pesi
      push 0x1 or 0x40000
      call pushContentLengthString
      db 'CONTENT_LENGTH=',0
      pushContentLengthString:
      push 0x1

    noContentLength:

    mov peax,pdword [pecx+8] ;(R/E)AX => status
    push peax ;push status

    ;if Status< 500 then type => integer else type=> string
    cmp peax,500
    mov eax,0x3 or 0x40000 ;type = integer
    jbe StatusIsInteger

      mov eax,0x1 or 0x40000 ;type => string

    StatusIsInteger:
    push peax ;push type
    call pushRedirectStatus
    db 'REDIRECT_STATUS=',0
    pushRedirectStatus:
    push 0x1

    ;get url from HTTPSession structure
    mov peax,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov peax,pdword [peax+psize*2] ;get url

    push peax
    push 0x1 or 0x40000
    call pushRequestedURI
    db 'REQUEST_URI=',0
    pushRequestedURI:
    push 0x1

    mov peax,pdword [pebx+psize*5] ;URLbuffer from HTTPSession

    push peax
    push 0x1 or 0x40000
    call pushScriptName
    db 'SCRIPT_NAME=',0
    pushScriptName:
    push 0x1

    ;get WWWroot
    push 0
    push 0x1 ;GET_FROM_CLIENTSESSION
    push CPO_NAME_PATH ;name
    call GetPluginDataPointerSave

    push peax
    push 0x1 or 0x20000 or 0x40000
    call pushDocumentRoot
    db 'DOCUMENT_ROOT=',0
    pushDocumentRoot:
    push 0x1

    ;get remote address
    push CSI_CLIENT_ADDRESS or CSI_ASCII_STRING
    call GetClientSessionInfoShort

    push peax
    push 0x1 or 0x20000 or 0x40000
    call pushRemoteAddr
    db 'REMOTE_ADDR=',0
    pushRemoteAddr:
    push 0x1

    ;get server address
    push CSI_LOCAL_ADDRESS or CSI_ASCII_STRING
    call GetClientSessionInfoShort

    push peax
    push 0x1 or 0x20000 or 0x40000
    call pushServerAddr
    db 'SERVER_ADDR=',0
    pushServerAddr:
    push 0x1

    ;get remote port
    push CSI_CLIENT_PORT or CSI_ASCII_STRING
    call GetClientSessionInfoShort

    push peax
    push 0x1 or 0x20000 or 0x40000
    call pushRemotePort
    db 'REMOTE_PORT=',0
    pushRemotePort:
    push 0x1

    ;get local port
    push CSI_LOCAL_PORT or CSI_ASCII_STRING
    call GetClientSessionInfoShort

    push peax
    push 0x1 or 0x20000 or 0x40000
    call pushServerPort
    db 'SERVER_PORT=',0
    pushServerPort:
    push 0x1

    ;get url from HTTPSession structure
    mov pesi,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov pesi,pdword [pesi+psize*2] ;get url

    ;find pointer to the question mark
    findQuestionMark:
      lodsb
      test al,al
      jz noQueryString

      cmp al,'?'
    jne findQuestionMark

      push pesi
      push 0x1 or 0x40000
      call pushQueryString
      db 'QUERY_STRING=',0
      pushQueryString:
      push 0x1

    noQueryString:
    call pushServerSoftware
    db "SERVER_SOFTWARE=",CGIServerName,0
    pushServerSoftware:
    push 0x1 or 0x40000

    call pushCGIInterface
    db "GATEWAY_INTERFACE=CGI/1.1",0
    pushCGIInterface:
    push 0x1 or 0x40000

    ;get httpVersion from HTTPSession structure
    mov peax,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov peax,pdword [peax] ;get httpVersion

    push peax
    push 0x1 or 0x40000
    call pushServerProtocol
    db 'SERVER_PROTOCOL=HTTP/',0
    pushServerProtocol:
    push 0x1

    ;get method from HTTPSession structure
    mov peax,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov peax,pdword [peax+psize] ;get method

    push peax
    push 0x1 or 0x40000
    call pushRequestMethod
    db 'REQUEST_METHOD=',0
    pushRequestMethod:
    push 0x1

    ;get pointer to first HTTP variable from HTTPSession structure
    mov pedi,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
    mov pedi,pdword [pedi] ;get httpVersion

    push pedi
    call strlen
    add pedi,peax ;skip to zero byte

    inc pedi

    ;get size of buffer
    mov ecx,dword [pebx+psize*7]   ;ECX => size to defaultVariables     \
    add pecx,pdword [pebx+psize*4] ;add begin pointer of buffer
    sub pecx,pedi                  ;sub current position of buffer to get the size left at current position

    loopHTTPVariables:
      cmp word [pedi],0x0D0A
      je NoHeaderVariables

      inc pedi ;skip 0x0D

      mov pesi,pedi ;(R/E)SI => variable name

      ;find value of HTTP variable
      xor eax,eax ;zero
      repnz scasb ;find zero byte
      jnz NoHeaderVariables

      inc pedi ;skip 0x0D

      ;push variable value
      push pedi
      push 0x1 or 0x40000

      call pushEqualSign
      db '=',0
      pushEqualSign:
      push 0x1

      ;push variable name
      push pesi
      push 0x1 or 0x10000

      call pushHTTP
      db 'HTTP_',0
      pushHTTP:
      push 0x1

      ;find next name of HTTP variable
      xor eax,eax ;zero
      repnz scasb ;find zero byte
      jnz NoHeaderVariables
    jmp loopHTTPVariables


    NoHeaderVariables:

    call CreateOneStringA

    pop pecx ;restore (R/E)CX
    mov pedi,peax ;save Enviroment

    push pesp                  ;ResponseHeader
    push pebx                  ;HTTPSession

    push pecx ;save (R/E)CX
    push pesi ;save (R/E)SI

    mov esi,0x1 ;GET_FROM_CLIENTSESSION
    mov eax,CPO_NAME_MAX_CGI_RUNTIME
    call GetPluginDataInt ;(R/E)CX => maximum allowed buffer size

    mov eax,ecx ;EAX => maximum allowed buffer size

    pop pesi ;restore (R/E)SI
    pop pecx ;restore (R/E)CX

    push peax                  ;Timeout

    push pecx ;save (R/E)CX
    push pesi ;save (R/E)SI

    mov esi,0x1 ;GET_FROM_CLIENTSESSION
    mov eax,CPO_NAME_MAX_SIZE_SEND
    call GetPluginDataInt ;(R/E)CX => maximum allowed buffer size

    mov eax,ecx ;EAX => maximum allowed buffer size

    pop pesi ;restore (R/E)SI
    pop pecx ;restore (R/E)CX

    push peax                  ;buffer size
    push pedi                  ;Enviroment
    push pecx                  ;CGI_path
    push pdword [pebp+psize*3] ;ClientSession
    push pdword [pebp+psize*2] ;ProtocolSession
    call LaunchCGI

    ;free enviroment
    push pedi
    call FreeMemLocation

    ;free the base path + URL
    push pedx
    call FreeMemLocation

    ;free the location
    push pecx
    call FreeMemLocation
    jmp closeHandleConnection

noCGIApplication:

  ;open a handle to the file
  push 0 ;flag => READ
  push pedx
  call openFileHandle
  mov pesi,peax ;(R/E)SI => file handle

  ;free the base path + URL
  push pedx
  call FreeMemLocation

  mov peax,pesi
  inc peax ;failed handle is -1, make it 0
  test peax,peax ;test on failed handle
  je send404Error

  ;search for the extension
  mov pedi,pdword [pebx+psize*5] ;URLbuffer

;-----Bewerk hier !!!!

  push pedi
  call strlen
  add pedi,peax  ;Get to the end of the request URL
  mov pecx,peax  ;ECX => size of the string

  ;Start seacrhing for the extension
  std ;Set search direction backwards
  mov al,'.' ;search for a dot
  repnz scasb ;start searching
  cld ;Set search direction forward
  jnz notFoundExtension

    inc pedi
    mov pdword [pesp+8+(psize*2)],pedi ;set contentTypeExtension
    mov pdword [pesp+8+(psize*1)],0 ;set unknown/binary if the extension has no mime-type
    or dword [pesp],FRH_USE_CONTENT_TYPE_EXTENSION ;add flag

  notFoundExtension:
  mov dword [pesp+8],200 ;set statusCode


  ;lock linked-list with internal protocol structures
  call GetInternalProtocolstructureReadLock

  ;get file size of the file
  if used is32
    push dword [pesp+8+(psize*3)] ;pointer to contentLength
  end if
  push pesi ;file handle
  call getfilesizeLongLong
  if used is64
    mov qword [rsp+8+(psize*3)],rax  ;set contentLength
  end if

  sendDataNotText:

  mov pedi,pesi ;(R/E)DI => file pointer

  mov pesi,pdword [pebx] ;DefaultHTTPVariables
  mov pesi,pdword [pesi+5*psize] ;range
  test pesi,pesi ;check if 'range' exists in the request header
  jz noRange
    lodsd ;get first four bytes
    cmp eax,'byte'
    jne noRange
    lodsw ;get two more bytes
    cmp ax,'s='
    jne noRange

      ;copy Content-length to contentRangeTotalLength
      if used is32
        mov peax,pesp
        push dword [peax+8+(psize*3)] ;Source = pointer to ContentLength
        push dword [peax+8+(psize*5)] ;Destination = pointer to contentRangeTotalLength
        call movQword
      else
        mov rax,qword [rsp+8+(psize*3)] ;RAX = ContentLength
        mov qword [rsp+8+(psize*5)],rax ;contentRangeTotalLength = ContentLength
      end if

      ;Get the first-byte-pos value
      ;push values for StrToInt or StrToIntQ(x86)
      if used is32
        push dword [pesp+8+(psize*4)] ;contentRangeOffset (pointer to 64-bit integer)
      end if
      push 10   ;base
      push pesi ;pointer to convert

      ;replace '-' with a zero byte
      findDash:
        lodsb
        test al,al
        jz errRange

        cmp al,'-'
       jnz findDash

       mov byte [pesi-1],0 ;set a zero byte where the dash was set

      if used is32
        call StrToIntQ
      else
        call StrToInt
        mov qword [rsp+8+(psize*4)],rax ;set contentRangeOffset
      end if

      mov byte [pesi-1],'-' ;set dash back

      ;check if contentRangeOffset=>contentRangeTotalLength
      if used is32
        mov peax,pesp
        ;if contentRangeOffset==ContentLength
        push dword [peax+8+(psize*5)] ;contentRangeTotalLength
        push dword [peax+8+(psize*4)] ;contentRangeOffset
        call cmpQword
        jz noRange

        ;if contentRangeOffset>ContentLength
        push dword [peax+8+(psize*5)] ;contentRangeTotalLength
        push dword [peax+8+(psize*4)] ;contentRangeOffset
        call cmpCarryQword
        jnc noRange

      else
        mov rax,qword [rsp+8+(psize*5)] ;RAX => contentRangeTotalLength
        cmp qword [rsp+8+(psize*4)],rax ;contentRangeOffset=>contentRangeTotalLength
        jae noRange
      end if

      ;is last-byte-pos represented?
      cmp byte [pesi],0
      je noLastBytePos

        ;convert noLastBytePos
        if used is32
          push 0 ;reserved
          ;create Qword
          push 0
          push 0

          push esp ;output qword (pointer to 64-bit integer)
        end if
        push 10   ;base
        push pesi ;pointer to convert

        if used is32
          call StrToIntQ
          mov peax,pesp

          mov ecx,dword [peax+8+0xC+(psize*5)] ;ECX => pointer to contentRangeTotalLength
          ;create a copy of contentRangeTotalLength
          push dword [pecx+4] ;high dword
          push dword [pecx]   ;low dword
          mov pecx,pesp ;ECX => pointer to copy of contentRangeTotalLength

          ;contentRangeOffset>=last-byte-pos
          push eax                        ;last-byte-pos
          push dword [peax+8+0xC+(psize*4)] ;contentRangeOffset
          call cmpCarryQword
          jae errRange2

          ;contentRangeTotalLength<contentRangeOffset
          push eax ;pointer to last-byte-pos
          push ecx ;pointer to copy of contentRangeTotalLength
          call SubQwordFromQword
          jb errRange2

          ;edit the contentLength with the "last-byte-pos"
          push ecx                         ;pointer to copy of (contentRangeTotalLength -last-byte-pos)
          push dword [peax+8+0xC+(psize*3)] ;pointer to contentLength
          call SubQwordFromQword

          add pesp,psize*5 ;remove thge 2 qwords and 1 reserved value

        else
          call StrToInt
          mov rcx,qword [rsp+8+(psize*5)] ;RCX => contentRangeTotalLength
          cmp qword [rsp+8+(psize*4)],rax ;contentRangeOffset>=last-byte-pos
          jae noRange
          sub rcx,rax ;contentRangeTotalLength<last-byte-pos
          jb noRange

          sub qword [rsp+8+(psize*3)],rcx ;edit the contentLength with the "last-byte-pos"
        end if

      noLastBytePos:
      if used is32
        mov peax,pesp
        ;edit the contentLength with the "first-byte-pos"
        push dword [peax+8+(psize*4)] ;contentRangeOffset
        push dword [peax+8+(psize*3)] ;pointer to contentLength
        call SubQwordFromQword
      else
        mov rax,qword [pesp+8+(psize*4)] ;RAX => contentRangeOffset
        sub qword [rsp+8+(psize*3)],rax  ;edit the contentLength with the "last-byte-pos"
      end if

      or dword [pesp],FRH_SEND_CONTENT_RANGE  ;set range flag
      mov dword [pesp+8],206 ;set statusCode
      jmp noRange

errRange2:
  ;remove copy of contentRangeTotalLength
  if used is32
    add pesp,psize*2
  end if

errRange:
  ;clear arguments for StrToIntQ/StrToInt from stack  (or the Qword with reserved dword)
  if used is32
    add pesp,psize*3
  else
    add rsp,psize*2
  end if

noRange:

  lea peax,pdword [pesp]

  ;send HTTP-header
  push DEFAULT_SENDTIMEOUT   ;timeout
  push peax                  ;ResponseHeader  (already pushed)
  push pdword [pebp+psize*3] ;ClientSession
  push pdword [pebp+psize*2] ;ProtocolSession
  call [SendHTTPHeader]

  mov peax,pdword [pebx]         ;(R/E)AX => defaultVariables
  mov peax,pdword [peax+psize]   ;EAX => method
  cmp dword [peax],'HEAD'        ;check if method is HEAD
  je dontSendFile

  mov esi,0x1 ;GET_FROM_CLIENTSESSION
  mov eax,CPO_NAME_MAX_SIZE_SEND
  call GetPluginDataInt ;(R/E)CX => maximum allowed buffer size

  mov peax,pesp ;(R/E)AX => stack

  push DEFAULT_SENDTIMEOUT         ;timeout

  test dword [peax],FRH_SEND_CONTENT_RANGE

  jz noRangeSet

    push pdword [peax+8+(psize*3)] ;size   (contentLength)
    push pdword [peax+8+(psize*4)] ;start  (contentRangeOffset)
    jmp rangeSet

noRangeSet:
    push 0                         ;size   (contentLength)
    push 0                         ;start  (contentRangeOffset)

rangeSet:
  push pecx                        ;bufferSize
  push pedi                        ;file
  push pdword [pebp+psize*3]       ;sess
  push pdword [pebp+psize*2]       ;psess
  call [SendFile]
  test eax,eax
  jnz dontSendFile

    mov dword [pesp],0 ;zero flags (

  dontSendFile:

  ;close file handle
  push pedi ;file handle
  call closefile

  closeHandleConnection:
  test pebx,pebx
  jz dontReleaseHTTPSession

    push pebx
    call [FreeHTTPSession]

  dontReleaseHTTPSession:
  test dword [pesp],FRH_CONNECTION_KEEP_ALIVE
jnz clientLoop

endHandleConnection:
call dleave
EndProc 2
ret (FunctionReturnFactor*2)

;------------------------------------------------------------------------------
;This function will search through a multicolumn list for one item and matches the domain name
;input:
;         EAX: Interger name of the list (The list will be retreined using GetPluginData function)
;     (R/E)BX: pointer to HTTPSession structure
;     (R/E)DI: pointer to the item to find

;
;output:
;    (R/E)SI => Row number
;    (R/E)CX => pointer to list
;    Zero flag: unset=succes set=unsuccful
;
;Corupts: R12 (x64), EAX

;------------------------------------------------------------------------------
SearchForDomainAndSpecificItem:
  if used is32
    push 0 ;create space on stack for the wild star domain name (*) redirect URL
  else
    xor r12,r12
  end if

  ;Find a posible redirection
  mov esi,0x1 or 0x4 or 0x8 ;GET_FROM_CLIENTSESSION or IS_LIST_POINTER or NO_COPY_LIST
  ;mov eax,CPO_NAME_REDIRECTIONS
  call GetPluginDataInt
  test eax,eax
  jz noMatchItemAndDomain
  ;(R/E)CX => pointer to multicolumn list

  xor esi,esi ;clear (R/E)SI to be the startFromRow parameter

lookForNextRow:
  ;look if if the requested URL can be found in the multicolumnlist
  push pesi                 ;startFromRow
  push 0                    ;flags
  push 0                    ;since it's not beeing or checked by this function for strings ... zero :)
  push pedi                 ;The string we are looking for
  push COLUMN_GENERAL_ITEM  ;colmn path
  push pecx                 ;pointer to list
  call [FindListItem]
  test peax,peax
  jz noMatchItemAndDomain ;jump if not redirection had been found
  mov pesi,peax ;(R/E)SI => rowId of found URL

  ;get host from HTTPSession structure
  mov peax,pdword [pebx] ;(R/A)EX => pointer to defaultVariables
  mov peax,pdword [peax+psize*3] ;get host
  test peax,peax
  jz checkGeneralDomainname ;jump if host is not send with the header

  ;check if the found URL has the same domain as requested
  push pesi                      ;startFromRow
  inc pdword [pesp]
  push FLI_COMPARE_ONLY_ONE_ROW  ;flags
  push 0                         ;since it's not beeing or checked by this function for strings ... zero :)
  push peax                      ;The string we are looking for
  push COLUMN_GENERAL_DOMAIN     ;colmn domain
  push pecx                      ;pointer to list
  call [FindListItem]
  test peax,peax
  jnz sendRedirect_delete_wild_start_URL_rowId ;the domain matches the requested domain, redirect the client to the URL

checkGeneralDomainname:
  ;check if the found URL matches domain *
  push pesi                      ;startFromRow
  inc pdword [pesp]
  push FLI_COMPARE_ONLY_ONE_ROW  ;flags
  push 0                         ;since it's not beeing or checked by this function for strings ... zero :)
  call pushGeneralDomainName     ;The string we are looking for
  db '*',0
  pushGeneralDomainName:
  push COLUMN_GENERAL_DOMAIN     ;colmn domain
  push pecx                      ;pointer to list
  call [FindListItem]
  test peax,peax
  jz lookForNextRow ;domain does not match, search for next matching URL

  if used is32
    mov dword [pesp],eax ;save the rowId of the URL used by the wild star
  else
    mov r12,rax         ;save the rowId of the URL used by the wild star
  end if

  jmp lookForNextRow

  sendRedirect_delete_wild_start_URL_rowId:
  if used is32
    pop eax
  end if
  ret

noMatchItemAndDomain:
  if used is32
    pop esi ;ESI => rowId of the URL with the wild star domain name (*)
  else
    mov rsi,r12 ;rSI => rowId of the URL with the wild star domain name (*)
  end if
  test pesi,pesi
ret

;------------------------------------------------------------------------------
;Same as shortSendData, but sends chunk size first!
;------------------------------------------------------------------------------
shortSendDataWithChunk:
call SendSizeChunk
call shortSendData
ret

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendData:
push pecx

push pesp                   ;len
push pesi                   ;buf
push DEFAULT_SENDTIMEOUT    ;timeout
push RSF_RETURN_ON_COMPLETE ;flags
push pdword [pebp+psize*3]  ;ClientSession
push pdword [pebp+psize*2]  ;ProtocolSession
call [SendData]

add pesp,psize
test eax,eax
ret

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => buffer
;    (R/E)CX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEAX:
push pesi

mov pesi,peax
call shortSendData

pop pesi
ret

;------------------------------------------------------------------------------
;Same as shortSendString, but sends chunk size first!
;------------------------------------------------------------------------------
shortSendDataEDXWithChunk:
if used EnableChunking
push pebx
mov pebx,shortSendDataWithChunk
jmp shortSendDataEDXEBXSet
end if

;------------------------------------------------------------------------------
;This function will send data
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)SI => buffer
;    (R/E)DX => size
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendDataEDX:
push pebx
mov pebx,shortSendData
shortSendDataEDXEBXSet:
push pecx

mov pecx,pedx
call pebx

pop pecx
pop pebx
ret

;------------------------------------------------------------------------------
;Same as shortSendString, but sends chunk size first!
;------------------------------------------------------------------------------
shortSendStringWithChunk:
if used EnableChunking
push pebx
mov pebx,shortSendDataWithChunk
jmp shortSendStringEBXSet
end if

;------------------------------------------------------------------------------
;This function will send a ASCII string
;input:
;    ebp+psize*2:ProtocolSession
;    ebp+psize*3:ClientSession
;    (R/E)AX => string
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
shortSendString:
push pebx
mov pebx,shortSendData

shortSendStringEBXSet:
push pecx
push pesi

mov pesi,peax

push pesi
call strlen
mov pecx,peax

call pebx

pop pesi
pop pecx
pop pebx
ret

;------------------------------------------------------------------------------
;This function will find an InteralProtocolStructure by a ProtocolSession
;input:
;1: ProtocolSession
;output:
;    (R/E)AX => pointer to a InteralProtocolStructure
;------------------------------------------------------------------------------
GetInternalProtocolStructure:
call denter

call GetInternalProtocolstructureReadLock

mov pedi,[cpyInternalProtocolstructure];(R/E)DI => pointer to first InternalProtocolStructure

loopThroughInternalProtocolStructureLinkedList:
  mov pesi,pedi ;(R/E)SI => pointer to current InternalProtocolStructure

  lodsp ;het pointer to the next InternalProtocolStructure
  mov pedi,peax ;(R/E)DI => next protocol structure

  lodsp ;get psess
  cmp peax,pdword [pebp+psize*2] ;is this the protocol we are looking for
  je foundPsess

  test pedi,pedi ;test is next InternalProtocolStructure is zero
jnz loopThroughInternalProtocolStructureLinkedList

xor esi,esi ;clear (R/E)SI
jmp endGetInternalProtocolStructure

foundPsess:
sub pesi,psize*2  ;substract by two members, since this was incremented buy the lods(d/q) calls

endGetInternalProtocolStructure:
call ReleaseInternalProtocolstructureReadLock

mov peax,pesi

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will get an read lock on the cpyInternalProtocolstructure linked list
;input:
;  none
;output:
;    none
;------------------------------------------------------------------------------
GetInternalProtocolstructureReadLock:
call denter

jmp FirstTyrReadLock

TryReadLockAgain:
  push 1
  call sleep

FirstTyrReadLock:
  if used is32
    ;EDX(high) EAX(low) => Qword to compair [cpyInternalProtocolstructureSignal] with
    xor edx,edx
    mov eax,[reading]

    ;ecx(high) ebx(low) => Qword to set [cpyInternalProtocolstructureSignal] to
    mov ebx,eax
    inc ebx ;increment the reading threads
    xor ecx,ecx
    lock cmpxchg8b [cpyInternalProtocolstructureSignal]
  else
    mov eax,[reading]   ;RAX => Qword to compair [cpyInternalProtocolstructureSignal] with
    mov ecx,eax         ;RCX => Qword to check [cpyInternalProtocolstructureSignal] to
    inc ecx             ;increment the ammount of 'reading' threads

    lock cmpxchg [cpyInternalProtocolstructureSignal], rcx
  end if
jnz TryReadLockAgain

call dleave
ret



;------------------------------------------------------------------------------
;This function will release the earlier gathered read lock on the cpyInternalProtocolstructure linked list
;input:
;  none
;output:
;    none
;------------------------------------------------------------------------------
ReleaseInternalProtocolstructureReadLock:
call denter

TryReleaseReadLockAgain:
  if used is32
    ;EDX(high) EAX(low) => Qword to compair [cpyInternalProtocolstructureSignal] with
    xor edx,edx
    mov eax,[reading]

    ;ecx(high) ebx(low) => Qword to set [cpyInternalProtocolstructureSignal] to
    mov ebx,eax
    dec ebx ;decrement the reading threads
    xor ecx,ecx ;set write lock
    lock cmpxchg8b [cpyInternalProtocolstructureSignal]
  else
    mov eax,[reading]   ;RAX => Qword to compair [cpyInternalProtocolstructureSignal] with
    mov ecx,eax         ;RCX => Qword to check [cpyInternalProtocolstructureSignal] to
    dec ecx             ;increment the ammount of 'reading' threads

    lock cmpxchg [cpyInternalProtocolstructureSignal], rcx
  end if
jnz TryReleaseReadLockAgain

call dleave
ret

;------------------------------------------------------------------------------
;This function will send chunk size
;input:
;    (R/E)CX Size Chunk
;    [ebp+psize*2]:ProtocolSession
;    [ebp+psize*3]:ClientSession
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
SendSizeChunk:
if used EnableChunking
push pedi
push pecx

sub pesp,0x20
mov pedi,pesp

mov byte [pedi],0x0D  ;begin chunk with new line
mov byte [pedi+1],0x0A
add pedi,2

push 0x10                 ;base at 16
push pedi                 ;output string
push pecx                 ;value
call IntToStr             ;next convert chunk size to string

push pedi
call strlen
mov byte [pedi+peax],0x0D   ;add another new line (for the data)
mov byte [pedi+peax+1],0x0A

sub pedi,2 ;set pointer to first new line
add peax,4 ;add lines to len

push peax

push pesp ;len
push pedi ;buf
push DEFAULT_SENDTIMEOUT     ;timeout
push RSF_RETURN_ON_COMPLETE  ;flags
push pdword [pebp+psize*3] ;ClientSession
push pdword [pebp+psize*2] ;ProtocolSession
call [SendData]

add pesp,0x20+psize ;buff with the len parameter

pop pecx  ;restore registers
pop pedi
end if
ret

;------------------------------------------------------------------------------
;This function will send the last chunk indicating the end of a stream
;input:
;    [ebp+psize*2]:ProtocolSession
;    [ebp+psize*3]:ClientSession
;output:
;    (R/E)AX => SendData return value
;    Zero flag: unset=succes set=unseccful
;------------------------------------------------------------------------------
sendLastChunk:
if used EnableChunking
push pesi
push pecx

mov pesi,nextChunkstr
mov ecx,endNextChunkstr-nextChunkstr
call shortSendData ;send (R/E)SI with size ECX

pop pecx
pop pesi
end if
ret

nextChunkstr db 0x0D, 0x0A, 0x30,0x0D, 0x0A,0x0D, 0x0A
endNextChunkstr:

if used is32
  zeroLength dq 0
end if
baseLocation db '/',0

;content200OK1 db "<HTML><HEAD><TITLE>200 OK</TITLE></HEAD><BODY><H1>Success</H1>We successfully talked the HTTP-protocol. The URL you requested is:"
;endContent200OK1:
;content200OK2 db "<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;endContent200OK2:

;if 32 bit, create an qword containing the size of the error
if used is32
  sizeContent413Error dq endContent413Error-content413Error
end if
content413Error db '<!DOCTYPE HTML><HTML><HEAD><meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1">', "<TITLE>413 Just too much!</TITLE></HEAD><BODY><H1>Less is more</H1>Your client just send too much data, keep your request clean and easy and try again.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
endContent413Error:

;if 32 bit, create an qword containing the size of the error
if used is32
  sizeContent404Error dq endContent404Error-content404Error
end if
content404Error db '<!DOCTYPE HTML><HTML><HEAD><meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1">', "<TITLE>404 Not Found</TITLE></HEAD><BODY><H1>My apology!</H1>The file you were looking for is unfortunately not found on this server.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
endContent404Error:


;--;if 32 bit, create an qword containing the size of the error
;--if used is32
;--  sizeContent408Error dq endContent408Error-content408Error
;--end if
;--content408Error db "<HTML><HEAD><TITLE>408 Took you long enough!</TITLE></HEAD><BODY><H1>Time is money</H1>It took too long for your client to transmit the request to the server. Time is money, so please speed up and try again.<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;--uendContent408Error:

;-;if 32 bit, create an qword containing the size of the error
;-if used is32
;-  sizeContent405Error dq endContent405Error-content405Error
;-end if
;-content405Error db "<HTML><HEAD><TITLE>405 Am I supposed to know the answer to everything?</TITLE></HEAD><BODY><H1>Can you reformulate</H1>The server is unable to understand the question you trying to ask. Can you please reformulate your question and try again?<HR><ADDRESS>", HtmlAddressNotation ,"</ADDRESS></BODY></HTML>"
;-endContent405Error: