
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;----------------------------------------------------
;                      RegMaxKey
;0x4  = hKey
;
;output: (R/E)AX = return value of RegQueryInfoKey
;        (R/E)DX = ammount of sub keys
;        (R/E)CX = length of the largest sub-key
;
;----------------------------------------------------

if used RegMaxKey
RegMaxKey:
push pebp
mov pebp,pesp
push pebx

push 0
mov pecx,pesp

push 0
mov pedx,pesp

mov pebx,pdword [pebp+psize*2] ;hKey

if used is64
  call denter
  MakeStack
end if

argpush 12,0
argpush 11,0
argpush 10,0
argpush 9,0
argpush 8,0
argpush 7,0
argpush 6,pecx ;lpcbMaxSubKeyLen
argpush 5,pedx ;lpcSubKeys
argpush 4,0
argpush 3,0
argpush 2,0
argpush 1,pebx ;hKey
call [RegQueryInfoKeyA]

if used is64
  call dleave
end if

pop pedx
pop pecx

pop pebx
mov pesp,pebp
pop pebp
ret psize
end if

;----------------------------------------------------
;                      RegAmmountOfKeys
;1: hKey
;2: SubKey
;3: SubSubKey
;4: SubSubSubKey
;5: SubSubSubSubKey
;
;output: (R/E)AX = ammount of sub keys
;
;----------------------------------------------------
if used RegAmmountOfKeys
RegAmmountOfKeys:
call denter

;open the register key
push pdword [pebp+psize*6]
push pdword [pebp+psize*5]
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
push 0
call OpenRegKey

xor edx,edx
test ecx,ecx   ;test return value
cmovnz eax,edx ;EAX = 0 if OpenRegKey failed
jnz endRegAmmountOfKeys

push peax ;hKey

push peax ;hKey
call RegMaxKey
if used is32
  mov ebx,edx
end if

call RegCloseKeycall

if used is32
  mov eax,ebx
else
  mov peax,pedx
end if

endRegAmmountOfKeys:
call dleave
ret psize*5
end if

;----------------------------------------------------
;                      RegAmmountOfValues
;1: hKey
;2: SubKey
;3: SubSubKey
;4: SubSubSubKey
;5: SubSubSubSubKey
;
;output: (R/E)AX = ammount of values
;
;----------------------------------------------------
if used RegAmmountOfValues
RegAmmountOfValues:
call denter

;open the register key
push pdword [pebp+psize*6]
push pdword [pebp+psize*5]
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
push 0
call OpenRegKey

xor edx,edx
test ecx,ecx   ;test return value
cmovnz eax,edx ;EAX = 0 if OpenRegKey failed
jnz endRegAmmountOfValues

push peax ;hKey

push peax ;hKey
call RegMaxValue
if used is32
  mov ebx,edx
end if

call RegCloseKeycall

if used is32
  mov eax,ebx
else
  mov peax,pedx
end if

endRegAmmountOfValues:
call dleave
ret psize*5
end if

;----------------------------------------------------
;                      RegMaxValue
;0x4  = hKey
;
;output: (R/E)AX = return value of RegQueryInfoKey
;        (R/E)DX = ammount of values in this key
;        (R/E)CX = length of the largest name of all the values
;        (R/E)SI = length of the largest value
;
;---------------------------------------------------
if used RegMaxValue
RegMaxValue:
push pebp
mov pebp,pesp
push pebx

push 0
mov pesi,pesp

push 0
mov pecx,pesp

push 0
mov pedx,pesp

mov pebx,pdword [pebp+psize*2] ;hKey

if used is64
  call denter
  MakeStack
end if

argpush 12,0
argpush 11,0
argpush 10,pesi ;lpcbMaxValueLen
argpush 9,pecx ;lpcbMaxValueNameLen
argpush 8,pedx ;lpcValues
argpush 7,0
argpush 6,0
argpush 5,0
argpush 4,0
argpush 3,0
argpush 2,0
argpush 1,pebx ;hKey
call [RegQueryInfoKeyA]

if used is64
  call dleave
end if

pop pedx
pop pecx
pop pesi

pop pebx
mov pesp,pebp
pop pebp
ret psize
end if

;----------------------------------------------------
;                      ReadRegDwordWithErrorJump
;1: hKey
;2: SubKey
;3: ValueName
;4: errorPointer
;
;----------------------------------------------------

if used ReadRegDwordWithErrorJump
ReadRegDwordWithErrorJump:
call denter

push pdword [pebp+psize*4]
push 0
push 0
push 0
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
call ReadRegEx
;  (r/e)cx = size of pointer (r/e)ax
;  (r/e)dx = return value of RegQueryValueEx
;  (r/e)ax = Memlocation with the read data
test edx,edx
jnz errReadRegDwordWithErrorJump

mov pecx,pdword [peax]

push peax
call FreeMemLocation

mov peax,pecx

call dleave
ret psize*4
end if

errReadRegDwordWithErrorJump:
call dleave
add pesp,psize*5
jmp pdword [pesp-psize]

;----------------------------------------------------
;                      ReadReg
;0x4  = hKey
;0x8  = SubKey
;0xC  = ValueName
;
;----------------------------------------------------

if used ReadReg
ReadReg:
push pebp
mov pebp,pesp

push pdword [pebp+psize*4]
push 0
push 0
push 0
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
call ReadRegEx

mov pesp,pebp
pop pebp
ret psize*3
end if

;------------

;redirect a call to RegCreateKey in 32bit, create a stack with arguments in 64 first.
if used RegCreateKeycall
;if used is32
;  RegCreateKeycall:
;  jmp [RegCreateKeyA]
;else
  RegCreateKeycall:
  call denter
  MakeStack

  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegCreateKeyA]

  call dleave
  ret psize*3
;end if
end if

;redirect a call to RegCloseKey in 32bit, create a stack with arguments in 64 first.
if used RegCloseKeycall | used SafeRegCloseKeycall
if used is32
  RegCloseKeycall:
  ;jmp [RegCloseKey]

  SafeRegCloseKeycall:
  call denter

  argpush 1,pdword [pebp+psize*2]
  call [RegCloseKey]

  call dleave
  ret psize
else
  SafeRegCloseKeycall:
  RegCloseKeycall:
  call denter
  MakeStack

  argpush 1,pdword [pebp+psize*2]
  call [RegCloseKey]

  call dleave
  ret psize
end if
end if

if used RegOpenKeyExcall
;redirect a call to RegOpenKeyEx in 32bit, create a stack with arguments in 64 first.
;if used is32
;  RegOpenKeyExcall:
;  jmp [RegOpenKeyExA]
;else
  RegOpenKeyExcall:
  call denter
  MakeStack

  mov peax,pdword [pebp+psize*6]

  argpush 5,peax
  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegOpenKeyExA]

  call dleave
  ret psize*5
;end if
end if

;redirect a call to RegQueryValueEx in 32bit, create a stack with arguments in 64 first.
if used RegQueryValueExcall
;if used is32
;  RegQueryValueExcall:
;  jmp [RegQueryValueExA]
;else
  RegQueryValueExcall:
  call denter
  MakeStack

  mov peax,pdword [pebp+psize*6]
  mov pecx,pdword [pebp+psize*7]

  argpush 6,pecx
  argpush 5,peax
  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegQueryValueExA]

  call dleave
  ret psize*6
;end if
end if

;redirect a call to RegSetValueEx in 32bit, create a stack with arguments in 64 first.
if used RegSetValueExcall
;if used is32
;  RegSetValueExcall:
;  jmp [RegSetValueExA]
;else
  RegSetValueExcall:
  call denter
  MakeStack

  mov peax,pdword [pebp+psize*6]
  mov pecx,pdword [pebp+psize*7]

  argpush 6,pecx
  argpush 5,peax
  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegSetValueExA]

  call dleave
  ret psize*6
;end if
end if

;redirect a call to RegEnumKey in 32bit, create a stack with arguments in 64 first.
if used RegEnumKeycall
;-if used is32
;-  RegEnumKeycall:
;-  jmp [RegEnumKeyA]
;-else
  RegEnumKeycall:
  call denter
  MakeStack

  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegEnumKeyA]

  call dleave
  ret psize*4
;-end if
end if

;redirect a call to RegDeleteKey in 32bit, create a stack with arguments in 64 first.
if used RegDeleteKeycall
;if used is32
;  RegDeleteKeycall:
;  jmp [RegDeleteKeyA]
;else
  RegDeleteKeycall:
  call denter
  MakeStack

  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegDeleteKeyA]

  call dleave
  ret psize*2
;end if
end if

;redirect a call to RegDeleteKey in 32bit, create a stack with arguments in 64 first.
if used RegEnumValuecall | used SafeRegEnumValuecall
if used is32
  RegEnumValuecall:
;  jmp [RegEnumValueA]

  SafeRegEnumValuecall:
  call denter

  mov peax,pdword [pebp+psize*6]
  mov pecx,pdword [pebp+psize*7]
  mov pedx,pdword [pebp+psize*8]
  mov pebx,pdword [pebp+psize*9]

  argpush 8,pebx
  argpush 7,pedx
  argpush 6,pecx
  argpush 5,peax
  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegEnumValueA]

  call dleave
  ret psize*8
else
  SafeRegEnumValuecall:
  RegEnumValuecall:
  call denter
  MakeStack

  mov rax,pdword [pebp+psize*6]
  mov rcx,pdword [pebp+psize*7]
  mov rdx,pdword [pebp+psize*8]
  mov rbx,pdword [pebp+psize*9]

  argpush 8,rbx
  argpush 7,rdx
  argpush 6,rcx
  argpush 5,rax
  argpush 4,pdword [pebp+psize*5]
  argpush 3,pdword [pebp+psize*4]
  argpush 2,pdword [pebp+psize*3]
  argpush 1,pdword [pebp+psize*2]
  call [RegEnumValueA]

  call dleave
  ret psize*8
end if
end if

;----------------------------------------------------
;                      OpenRegKey
;0x4  = CreateKey
;0x8  = hKey
;0xC  = SubKey
;0x10 = SubSubKey
;0x14 = SubSubSubKey
;0x18 = SubSubSubSubKey
;
;output
;  (r/e)cx = return value of RegOpenKeyEx
;  (r/e)ax = hKey
;----------------------------------------------------

if used OpenRegKey
OpenRegKey:
push pebp
mov pebp,pesp
push pedx
push pesi
push pedi

mov pedx,pdword [pebp+psize*3]
mov pesi,pebp
add pesi,psize*4

FindFinalKey:
mov pecx,pdword [pebp+psize*2]
jpcxz DoNotCreateKey
push pesi ;save the position of the name of the current subKey
push pedx ;save hKey

;create room on stack for new hKey
push 0
mov pecx,pesp

push pecx
push pdword [pesi]
push pedx
call RegCreateKeycall

call RegCloseKeycall

pop pedx  ;restore hKey
pop pesi  ;restore the position of the name of the current subKey
DoNotCreateKey:
;create room on stack for new hKey
push 0
mov pecx,pesp

push pesi ;save the position of the name of the current subKey
push pedx ;save 'old' hKey

push pecx           ;retv
push KEY_ALL_ACCESS ;acces
push 0
push pdword [pesi]  ;subKey
push pedx           ;hKey
call RegOpenKeyExcall

pop pedx  ;pop the old key
cmp pdword [pebp+psize*3],pedx   ;is the key a predefined hkey?
je DontCloseThisOne
push peax ;save return value of RegOpenKeyEx

;free the key if not predefined
push pedx
call RegCloseKeycall

pop peax  ;restore return value

DontCloseThisOne:
pop pesi   ;restore the position of the name of the current subKey
pop pedx   ;the new hKey
add pesi,psize

mov pedi,pebp
add pedi,8*psize
cmp pesi,pedi
je HasFinalKey

cmp pdword [pesi],0
jne FindFinalKey
;--

HasFinalKey:
xchg pedx,peax
mov pecx,pedx

pop pedi
pop pesi
pop pedx
mov pesp,pebp
pop pebp
ret 6*psize
end if


;----------------------------------------------------
;                      ReadRegEx
;0x4  = hKey
;0x8  = SubKey
;0xC  = SubSubKey
;0x10 = SubSubSubKey
;0x14 = SubSubSubSubKey
;0x18 = ValueName
;
;output
;  (r/e)cx = size of pointer (r/e)ax
;  (r/e)dx = return value of RegQueryValueEx
;  (r/e)ax = Memlocation with the read data
;----------------------------------------------------

if used ReadRegEx
ReadRegEx:
push pebp
mov pebp,pesp

;open the register key
push pdword [pebp+psize*6]
push pdword [pebp+psize*5]
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
push 0
call OpenRegKey
xchg peax,pecx  ;switch hKey and return value of RegOpenKeyEx

push peax ;save return value of RegOpenKeyEx
push pecx ;save hKey

;push pecx
;call RegMaxValue

;get the size of the item we want to read
push 0

push pesp                   ;buffer size
push 0                      ;buffer
push 0
push 0
push pdword [pebp+psize*7]  ;valueName
push pecx                   ;hKey
call RegQueryValueExcall

pop pesi      ;(R/E)SI = size of the item
push pesi     ;save the size of the item
add pesi,0x10 ;add 10 bytes to be sure

;allocate memory to read the item out
push pesi
call GetMemLocation
mov pecx,pesp  ;pointer to the size of the item
mov pedx,pdword [pesp+psize]  ;restore the hKey
push peax  ;push memory location

;read the value from the registery
push pecx                   ;buffer size
push peax                   ;buffer
push 0
push 0
push pdword [pebp+psize*7]  ;valueName
push pedx                   ;hKey
call RegQueryValueExcall
push peax  ;push return value of RegQueryValueEx

;close the register key
push pdword [pesp+psize*3]
call RegCloseKeycall

pop pedx  ;RegQueryValueEx return value
pop peax  ;Memlocation
pop pecx  ;size of the memlocation
add pesp,psize*2
mov pesp,pebp
pop pebp
ret psize*6

end if


;----------------------------------------------------
;                      WriteRegString
;0x4  = SubKey
;0x8  = ValueName
;0xC  = ValueData
;0x10 = ValueDataLen
;
;----------------------------------------------------

if used WriteRegString
WriteRegString:
call denter

push REG_SZ
push pdword [pebp+psize*5]
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push 0
push 0
push 0
push pdword [pebp+psize*2]
if used is64
  mov peax,HKEY_LOCAL_MACHINE
  push peax
else
  push HKEY_LOCAL_MACHINE ;use key Local Machine
end if
call WriteRegEx

call dleave
ret psize*4
end if

;----------------------------------------------------
;                      WriteRegDword
;0x4  = SubKey
;0x8  = ValueName
;0xC  = ValueData
;
;----------------------------------------------------

if used WriteRegDword
WriteRegDword:
call denter

push REG_DWORD
push 4
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push 0
push 0
push 0
push pdword [pebp+psize*2]
if used is64
  mov peax,HKEY_LOCAL_MACHINE
  push peax
else
  push HKEY_LOCAL_MACHINE ;use key Local Machine
end if
call WriteRegEx

call dleave
ret psize*3
end if

;----------------------------------------------------
;                      WriteRegBinary
;0x4  = SubKey
;0x8  = ValueName
;0xC  = ValueData
;0x10 = ValueDataLen
;
;----------------------------------------------------

if used WriteRegBinary
WriteRegBinary:
call denter

push REG_BINARY
push pdword [pebp+psize*5]
push pdword [pebp+psize*4]
push pdword [pebp+psize*3]
push 0
push 0
push 0
push pdword [pebp+psize*2]
if used is64
  mov peax,HKEY_LOCAL_MACHINE
  push peax
else
  push HKEY_LOCAL_MACHINE ;use key Local Machine
end if
call WriteRegEx

call dleave
ret psize*4
end if

;----------------------------------------------------
;                      WriteRegEx
;0x4  = hKey
;0x8  = SubKey
;0xC  = SubSubKey
;0x10 = SubSubSubKey
;0x14 = SubSubSubSubKey
;0x18 = ValueName
;0x1C = ValueData
;0x20 = ValueDataLen
;0x24 = ValueDataType
;
;----------------------------------------------------

if used WriteRegEx
WriteRegEx:
call denter

push pdword [pebp+psize*6]  ;SubSubSubSubKey
push pdword [pebp+psize*5]  ;SubSubSubKey
push pdword [pebp+psize*4]  ;SubSubKey
push pdword [pebp+psize*3]  ;SubKey
push pdword [pebp+psize*2]  ;hKey
push 1                      ;create the key
call OpenRegKey
xchg peax,pecx

push pecx ;push for RegCloseKey

push pdword [pebp+psize*9]  ;ValueDataLen
push pdword [pebp+psize*8]  ;ValueData
push pdword [pebp+psize*10] ;ValueDataType
push 0
push pdword [pebp+psize*7]  ;ValueName
push pecx
call RegSetValueExcall

call RegCloseKeycall

call dleave
ret 9*psize
end if

;----------------------------------------------------
;                      EnumRegKey
;1: hKey
;2: SubKey
;3: SubSubKey
;4: SubSubSubKey
;5: SubSubSubSubKey
;6: CallBackFunction:
;               arg: 1:hKey
;                    2:param
;                    3:Keyname
;7: Param for callbackfunction
;
;----------------------------------------------------
if used EnumRegKey
EnumRegKey:
call denter

push pdword [pebp+psize*6]  ;SubSubSubSubKey
push pdword [pebp+psize*5]  ;SubSubSubKey
push pdword [pebp+psize*4]  ;SubSubKey
push pdword [pebp+psize*3]  ;SubKey
push pdword [pebp+psize*2]  ;hKey
push 0                      ;create the key
call OpenRegKey
test pecx,pecx
jnz NoSubKeysEnumRegKey
mov pebx,peax

push pebx
call RegMaxKey

test pedx,pedx
jz NoSubKeysEnumRegKey
xchg pedx,pecx ;EDX = max length sub-key ECX = ammount of sub-keys

inc pedx ;add terminating zero byte

push pedx ;max length of a sub-key
call GetMemLocation
mov pedi,peax

dec pecx
xor pesi,pesi ;set index  on esi

if used is32
  push pedx ;save max length of a sub-key
end if

loopSubKeys:
   if used is32
     mov edx,dword [pesp]
   end if

  push pecx ;safe counter

  push pedx                   ;Length
  push pedi                   ;Str
  push pesi                   ;Index
  push pebx                   ;Regkey
  call RegEnumKeycall

  push 0     ;SubSubSubSubKey
  push 0     ;SubSubSubKey
  push 0     ;SubSubKey
  push pedi  ;SubKey
  push pebx  ;hKey
  push 0     ;create the key
  call OpenRegKey
  test pecx,pecx
  jnz nextLoopSubKeys

  push peax ;push hKey for RegCloseKey

  push pedi ;name of the key
  push pdword [pebp+psize*8] ;param
  push peax ;hKey
  call pdword [pebp+psize*7] ;callBack function

  call RegCloseKeycall ;close key

nextLoopSubKeys:
  pop pecx
  inc pesi
  sub pecx,1
jnc loopSubKeys

;delete temperary string
push pedi
call FreeMemLocation

;close registery key
push pebx
call RegCloseKeycall

if used is32
  add pesp,psize
end if

NoSubKeysEnumRegKey:
call dleave
ret psize*7
end if

;----------------------------------------------------
;                      EnumRegValue
;1: hKey
;2: SubKey
;3: SubSubKey
;4: SubSubSubKey
;5: SubSubSubSubKey
;6: CallBackFunction:
;               arg: 1:hKey
;                    2:param
;                    3:valueName
;                    4:value
;                    5:valueSize
;                    6:valueType
;7: Param for callbackfunction
;
;----------------------------------------------------
if used EnumRegValue
EnumRegValue:
call denter

push pdword [pebp+psize*6]  ;SubSubSubSubKey
push pdword [pebp+psize*5]  ;SubSubSubKey
push pdword [pebp+psize*4]  ;SubSubKey
push pdword [pebp+psize*3]  ;SubKey
push pdword [pebp+psize*2]  ;hKey
push 0                      ;create the key
call OpenRegKey
test pecx,pecx
jnz endEnumRegValue
mov pebx,peax

push pebx
call RegMaxValue

;        (R/E)DX = ammount of values in this key
;        (R/E)CX = length of the largest name of all the values
;        (R/E)SI = length of the largest value

test pedx,pedx
jz endEnumRegValue
xchg pedx,pecx ;EDX = max length the largest name of all the values ECX = ammount of values

inc pedx ;add terminating zero byte

push pedx ;max length of a sub-key
call GetMemLocation
mov pedi,peax ;buffer for the value name

push pesi ;ength of the largest value
call GetMemLocation

push pedx  ;backup length str
push pedx  ;length str
mov pedx,pesp

;pre push the length of the value buffer and the buffer
push pesi ;backup buffer size
push pesi ;buffer size

push pesp ;pointer to buffer size
push peax ;buffer

dec pecx
xor esi,esi ;set index  on esi

loopValues:
   if used is32
     mov pedx,pesp
     add edx,psize*4
   end if

  ;push size
  ;push buffer
  push 0                      ;Type
  push 0                      ;Reserved
  push pedx                   ;Length
  push pedi                   ;Str
  push pesi                   ;Index
  push pebx                   ;Regkey
  call SafeRegEnumValuecall

  if used is32
    mov pedx,pesp
    add pedx,psize*2
  end if

  ;restore length
  mov peax,pdword [pedx+psize]
  mov pdword [pedx],peax

  ;restore size
  mov peax,pdword [pesp+psize]
  mov pdword [pesp],peax

  sub pesp,psize*2 ;restore the buffer and size

  ;get the type of the item
  push 0
  mov peax,pesp

  ;get the real size of the item (instead of the max size)
  push 0

  push pesp                   ;&size
  push 0                      ;buffer
  push peax                   ;Type
  push 0                      ;Reserved
  push pedx                   ;Length
  push pedi                   ;Str
  push pesi                   ;Index
  push pebx                   ;Regkey
  call SafeRegEnumValuecall

   if used is32
     mov pedx,pesp
     add pedx,psize*6
   end if

  ;restore length
  mov peax,pdword [pedx+psize]
  mov pdword [pedx],peax

  ;push buffer
  mov peax,pdword [pesp+psize*2]
  push peax

  ;push type                 ;value type
  ;push bufferSize           ;value size
  ;push buffer               ;value
  push pedi                  ;name of the value
  push pdword [pebp+psize*8] ;param
  push pebx ;hKey
  call pdword [pebp+psize*7] ;callBack function

nextloopValues:
  inc pesi
  sub pecx,1
jnc loopValues

;delete temperary string
push pedi
call FreeMemLocation

;delete the pushed buffer
call FreeMemLocation

;close registery key
push pebx
call RegCloseKeycall

add pesp,psize*4 ;remove the size en pointer to size from stack and size of the other buffer

endEnumRegValue:
call dleave
ret psize*7
end if

;----------------------------------------------------
;This function deletes the a value from the registry
;input:
;  1: hKey
;  2: SubKey
;  3: SubSubKey
;  4: SubSubSubKey
;  5: SubSubSubSubKey
;  6: value name
;
;----------------------------------------------------

if used DeleteRegValue
DeleteRegValue:
call denter
MakeStack

;open registry key
push pdword [pebp+psize*6]  ;SubSubSubSubKey
push pdword [pebp+psize*5]  ;SubSubSubKey
push pdword [pebp+psize*4]  ;SubSubKey
push pdword [pebp+psize*3]  ;SubKey
push pdword [pebp+psize*2]  ;hKey
push 0                      ;create the key
call OpenRegKey
test pecx,pecx
jnz endDeleteRegValue
mov pebx,peax

;delete value
argpush 2,pdword [pebp+psize*7]
argpush 1,pebx
call [RegDeleteValueA]

;close registery key
push pebx
call RegCloseKeycall

endDeleteRegValue:
call dleave
ret psize*6
end if

;----------------------------------------------------
;This function deletes the sub keys of the key to deletem afther that it will delete the key itself
;input
;  1: hKey
;  2: SubKey
;
;----------------------------------------------------

if used DeleteRegKey
DeleteRegKey:
call denter

push 0
push 0
push 0
push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
push 0
call OpenRegKey
mov pebx,peax

push peax
call RegMaxKey

test pedx,pedx
jz NoSubKeys
xchg pedx,pecx ;EDX = max length sub-key ECX = ammount of sub-keys

inc pedx ;add terminating zero byte

push pedx ;max length of a sub-key
call GetMemLocation
mov pesi,peax     ;fix this

dec pecx

RemovSubKeys:
  push pecx

  push pedx                   ;Length
  push pesi                   ;Str
  push pecx                   ;Index
  push pebx                   ;Regkey
  call RegEnumKeycall

  push pesi
  push pebx
  call DeleteRegKey

  pop pecx
  jpcxz NoItemsAnnymore
  dec pecx
  jmp RemovSubKeys

NoItemsAnnymore:
push pesi
call FreeMemLocation

NoSubKeys:
push pebx
call RegCloseKeycall

push pdword [pebp+psize*3]
push pdword [pebp+psize*2]
call RegDeleteKeycall

call dleave
ret psize*2
end if