
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

;Functions :
;InstallService
;UninstallService
;doSystemCheck
;getRebootAcces
;StopService
;StopDeleteService
;SetServiceState
;GetServiceState
;startService
;IsRunningInteractive
;CheckAndStartService
;service_main
;ServiceProc

RebootMsg db "The service " , Name , " has failed to start. The computer will reboot in 1 minute in order to resolve this problem.",0
SE_SHUTDOWN_NAME db "SeShutdownPrivilege",0,0

service_name db ServiceName,0
service_description_string db ServiceDescription,0

service_dependencies db 'Tcpip',0,'Afd',0,0

advapi32_name db 'advapi32.dll',0

ServiceTable:
pointer Handle
pointer service_main
pointer 0
pointer 0

;SERVICE_DESCRIPTION structure
service_description:
pointer service_description_string

;SC_ACTION structure
service_actions:
dd 1         ;SC_ACTION_RESTART
dd 1000*60   ;DELAY
dd 1         ;SC_ACTION_RESTART
dd 1000*60   ;DELAY
dd 1         ;SC_ACTION_RESTART
dd 1000*60   ;DELAY
;-dd 2         ;SC_ACTION_REBOOT
;-dd 1000*60   ;DELAY

;SERVICE_FAILURE_ACTIONS   structure
service_failure_actions_struct:
pointer 60*60*24        ;dwResetPeriod
pointer RebootMsg       ;lpRebootMsg
pointer 0               ;lpCommand
pointer 3               ;cActions
pointer service_actions ;lpsaActions

Handle pointer 0

if used is32
  is9x db 0
  regKeyServices9x db 'SOFTWARE\Microsoft\Windows\CurrentVersion\RunServices',0
end if

;------------------------------------------------------------------------------
;This function will check if the process runs on windows 9x or NT,
;since 9x x64 does not exist, this function does nothing on x64. This
;function should be direct at the entrypoint.
;
;input:
;   none
;output:
;    none
;------------------------------------------------------------------------------
if used is32
doSystemCheck:
  mov eax,dword [fs:0x30] ;Get PEB from TEB
  cmp ebx,eax
  je IsNT

  inc [is9x]

  IsNT:
ret
end if

;------------------------------------------------------------------------------
;This function will install the service on Windows NT based systems
;input:
;   none
;output:
;    (R/E)AX=> Boolean, true on succes and false of failure
;------------------------------------------------------------------------------
InstallService:
call denter

;create room on stack for the filename of this application  and zero this string
sub pesp,288
mov pebx,pesp

MakeStack

call getRebootAcces

inc pebx

;call GetModuleFileName to receive the path to thie application
argpush 3,260
argpush 2,pebx
argpush 1,0
call [GetModuleFileNameA]

if used is32
  ;In windows 9x you need to set a register value to install a service
  test [is9x],1
  jz isNTSystem2
    ;calculate the size of the module file name
    push pebx
    call strlen

    ;set string in registry to set this service as bootable service
    push eax
    push pebx
    push service_name
    push regKeyServices9x
    call WriteRegString

    jmp endInstallService

  isNTSystem2:
end if

;add a quote-char at the beginning the of filename
dec pebx
mov pedi,pebx
mov byte [pedi],'"'

;add a end the of filename and add a quote there
;--add pedi,260

mov ecx,260  ;repeat search for 260 bytes (the max for the path string)
xor eax,eax  ; byte to find with scasb => '\0'

repnz scasb   ;loop string until a zero byte is found

;add quote at the end of the filename
mov word [pedi-1],'"'

;open a SC manager with all rights
argpush 3,0x2 ;SC_MANAGER_CREATE_SERVICE / SERVICE_CHANGE_CONFIG ;0xF003F
argpush 2,0
argpush 1,0
call [OpenSCManagerA]
test peax,peax
jz endInstallService

mov pesi,peax ;(R/E)SI => result of OpenSCManagerA


argpush 13,0                     ;lpPassword
argpush 12,0                     ;lpServiceStartName
argpush 11,service_dependencies  ;lpDependencies
argpush 10,0                     ;lpdwTagId
argpush 9,0                      ;lpLoadOrderGroup
argpush 8,pebx                   ;lpBinaryPathName
argpush 7,0                      ;dwErrorControl
argpush 6,0x2                    ;dwStartType
argpush 5,0x10                   ;dwServiceType
argpush 4,0xF003F                ;dwDesiredAccess
argpush 3,service_name           ;lpDisplayName
argpush 2,service_name           ;lpServiceName
argpush 1,pesi                   ;hSCManager
call [CreateServiceA]
test peax,peax
jz CreateServiceFailed

mov pebx,peax  ;save result of CreateService on (R/E)BX

;Windows NT does not support function ChangeServiceConfig2A, so see if it exist in x86
if used is32
  ;Try to  receive pointer from function 'ChangeServiceConfig2A', since it does not exist in Windows NT
  push advapi32_name
  call loadmodule

  call pushFunction
  db 'ChangeServiceConfig2A',0
  pushFunction:
  push peax
  call getfunc
  test peax,peax
  je runningOldSystem
end if

if used is32
  push eax    ;save pointer to ChangeServiceConfig2A function
end if

argpush 3,service_description  ;pointer to a SERVICE_DESCRIPTION structure
argpush 2,1                    ;SERVICE_CONFIG_DESCRIPTION
argpush 1,pebx                 ;hService
if used is32
  call peax     ;call ChangeServiceConfig2A
else
  call [ChangeServiceConfig2A]
end if

if used is32
  pop eax  ;restore pointer to ChangeServiceConfig2A function
end if

argpush 3,service_failure_actions_struct ;SERVICE_FAILURE_ACTIONS structure
argpush 2,2                              ;SERVICE_CONFIG_FAILURE_ACTIONS
argpush 1,pebx                           ;hService
if used is32
  call peax     ;call ChangeServiceConfig2A
else
  call [ChangeServiceConfig2A]      ;call ChangeServiceConfig2A
end if

runningOldSystem:
xor eax,eax
inc eax ;return 1 since the function succeeded

argpush 1,pebx
call [CloseServiceHandle]

CreateServiceFailed:
argpush 1,pesi
call [CloseServiceHandle]

endInstallService:
call dleave
ret

;------------------------------------------------------------------------------
;This function will request reboot acces to be able to set the service failure actions to SC_ACTION_REBOOT.
;input:
;   none
;output:
;    (R/E)AX=> Boolean, true on succes and false of failure
;------------------------------------------------------------------------------
getRebootAcces:
call denter

push 0 ;attributes
push 0 ;LUID
push 0 ;LUID
push 0 ;PrivilegeCount / hToken
mov pebx,pesp
MakeStack

call [GetCurrentProcess]

argpush 3,pebx ;hToken
argpush 2,0x00000020 or 0x00000008;TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY
argpush 1,peax
call [OpenProcessToken]
test eax,eax
jz endGetRebootAcces

mov pesi,pdword [pebx] ;(R/E)SI => token

lea peax,pdword [pebx+4] ;get pointer to LUID structure on stack

argpush 3,peax;prv
argpush 2,SE_SHUTDOWN_NAME
argpush 1,0
call [LookupPrivilegeValueA]   ;creates a new thread on Windows 7, kill it !!!!
test eax,eax
jz LookupPrivilegeFailed

mov pecx,pesp

mov dword [pebx],1     ;set PrivilegeCount in structure TOKEN_PRIVILEGES
mov dword [pebx+0xC],2 ;Set attributes in structure _LUID_AND_ATTRIBUTES  to SE_PRIVILEGE_ENABLED

argpush 6,0
argpush 5,0
argpush 4,0
argpush 3,pebx;xtkp;tkp
argpush 2,0
argpush 1,pesi
call [AdjustTokenPrivileges]

LookupPrivilegeFailed:
mov ebx,eax ;save return value

argpush 1,pesi
call [CloseHandle]

mov eax,ebx ;restore return value

endGetRebootAcces:
call dleave
ret

;------------------------------------------------------------------------------
;This function will uninstall the service on Windows NT-based systems
;input:
;   none
;output:
;    (R/E)AX=> Boolean, true on succes and false of failure
;------------------------------------------------------------------------------
UninstallService:
if used is32
  ;In windows 9x a register value is set to install the server, delete this key to uninstall the service
  test [is9x],1
  jz isNTSystem3
    ;Delete register value
    push service_name
    push 0
    push 0
    push 0
    push regKeyServices9x
    push HKEY_LOCAL_MACHINE
    call DeleteRegValue
    mov edi,eax

    jmp endUninstallService

  isNTSystem3:
end if

push 1
call StopDeleteService

endUninstallService:
ret

;------------------------------------------------------------------------------
;This function will stop the service if running on Windows NT-based systems
;input:
;   none
;output:
;    (R/E)AX=> Boolean, true on succes and false of failure
;------------------------------------------------------------------------------
StopService:
push 0
call StopDeleteService
ret

;------------------------------------------------------------------------------
;This function will stop the service if running and uninstall if the uninstall flag is set on Windows NT-based systems
;input:
;   1:flags:
;       0x0 STOP_SERVICE
;       0x1 STOP_AND_DELETE_SERVICE
;output:
;    (R/E)AX=> Boolean, true on succes and false of failure
;------------------------------------------------------------------------------
StopDeleteService:
call denter
MakeStack

argpush 3,0x10000 or 0x20 ;DELETE or SERVICE_STOP
argpush 2,0
argpush 1,0
call [OpenSCManagerA]
test peax,peax
jz endStopDeleteService

mov pesi,peax ;(R/E)SI => result of OpenSCManagerA

;open handle to service TV's server v3
argpush 3,0x10000 or 0x20  ;DELETE or SERVICE_STOP
argpush 2,service_name     ;name of the service to open
argpush 1,peax             ;hServiceManager
call [OpenServiceA]
test peax,peax
jz OpenServiceFailed

mov pebx,peax;(R/E)BX => handle to service

;create a SERVICE_STATUS structure
push 0      ;dwWaitHint
push 0      ;dwCheckPoint
push 0      ;dwServiceSpecificExitCode
push 0      ;dwWin32ExitCode
push 0      ;dwControlsAccepted
push 0      ;dwCurrentState
push 0      ;dwServiceType
mov pecx,pesp

if used is64
  sub rsp,5*8 ;create space on stack for calling ControlService, DeleteService and CloseServiceHandle  with Windows Calling Convention x64
end if

argpush 3,pecx  ;pointer to a SERVICE_STATUS structure
argpush 2,0x1   ;command to stop the service
argpush 1,pebx  ;hService
call [ControlService]

test pdword [pebp+psize*2],1 ;Delete the service?
jz dontDeleteService

  argpush 1,pebx
  call [DeleteService]

dontDeleteService:
;close handle to service
argpush 1,pebx
call [CloseServiceHandle]

;-if used is64
;-  add rsp,5*8 ;remove space on stack for calling ControlService, DeleteService and CloseServiceHandle with Windows Calling Convention x64
;-end if

;-add pesp,7*psize ;remove SERVICE_STATUS from stack

;set return value on true (one)
xor eax,eax
inc eax

OpenServiceFailed:
mov ebx,eax ;save return value

;close handle to service manager
argpush 1,pesi
call [CloseServiceHandle]

mov eax,ebx ;restore return value

endStopDeleteService:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will set the status of the service (that this process is currently running)
;input:
;   1:New status
;output:
;    (R/E)AX=> boolean: true if succeed
;
;New status:
;       0x1 SERVICE_STOPPED The service is not running.
;       0x2 SERVICE_START_PENDING The service is starting.
;       0x3 SERVICE_STOP_PENDING The service is stopping.
;       0x4 SERVICE_RUNNING The service is running.
;       0x5 SERVICE_CONTINUE_PENDING The service continue is pending.
;       0x6 SERVICE_PAUSE_PENDING The service pause is pending.
;       0x7 SERVICE_PAUSED The service is paused.
;------------------------------------------------------------------------------
SetServiceState:
call denter

;create an SERVICE_STATUS structure
sub pesp,0x20

mov dword [pesp+0x18],1000   ;dwWaitHint
mov dword [pesp+0x14],0      ;dwCheckPoint
mov dword [pesp+0x10],0      ;dwServiceSpecificExitCode
mov dword [pesp+0xC],0       ;dwWin32ExitCode
mov dword [pesp+0x8],0x1     ;dwControlsAccepted = SERVICE_ACCEPT_STOP
mov eax,dword [pebp+psize*2]
mov dword [pesp+0x4],eax     ;dwCurrentState
mov dword [pesp],0x10        ;dwServiceType = SERVICE_WIN32_OWN_PROCESS
mov pecx,pesp

MakeStack

argpush 2,pecx
argpush 1,[Handle]
call [SetServiceStatus]

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will return the status of the current external running process
;input:
;   none
;output:
;    (R/E)AX=>
;       0x0 NOT_INSTALLED The service is not installed or we have no privillage to get the service status
;       0x1 SERVICE_STOPPED The service is not running.
;       0x2 SERVICE_START_PENDING The service is starting.
;       0x3 SERVICE_STOP_PENDING The service is stopping.
;       0x4 SERVICE_RUNNING The service is running.
;       0x5 SERVICE_CONTINUE_PENDING The service continue is pending.
;       0x6 SERVICE_PAUSE_PENDING The service pause is pending.
;       0x7 SERVICE_PAUSED The service is paused.
;------------------------------------------------------------------------------
if used GetServiceState
GetServiceState:
call denter

sub pesp,7*4 ;creat a SERVICE_STATUS structure
mov pedi,pesp ;(R/E)DI => pointer to a SERVICE_STATUS structure
mov dword [pedi+4],0 ;set currentstatus to zero (not installed)

MakeStack

argpush 3,0x4             ;SERVICE_QUERY_STATUS
argpush 2,0
argpush 1,0
call [OpenSCManagerA]
test peax,peax
jz endGetServiceState

mov pesi,peax ;(R/E)SI => result of OpenSCManagerA

;open handle to service TV's server v3
argpush 3,0x4              ;SERVICE_QUERY_STATUS
argpush 2,service_name     ;name of the service to open
argpush 1,peax             ;hServiceManager
call [OpenServiceA]
test peax,peax
jz OpenServiceFailed2

mov pebx,peax;(R/E)BX => handle to service

argpush 2,pedi
argpush 1,pebx
call [QueryServiceStatus]

;close handle to service
argpush 1,pebx
call [CloseServiceHandle]

OpenServiceFailed2:
;close handle to service manager
argpush 1,pesi
call [CloseServiceHandle]

endGetServiceState:
mov eax,dword [pedi+4] ;EAX => currentstatus
call dleave
ret
end if

;------------------------------------------------------------------------------
;This function will try to boot the service
;input:
;   none
;output:
;    (R/E)AX=>  boolean, true if succeed, else false
;------------------------------------------------------------------------------
startService:
call denter
MakeStack

argpush 3,0x0010 ;SERVICE_START  0x20000000             ;SERVICE_QUERY_STATUS
argpush 2,0
argpush 1,0
call [OpenSCManagerA]
test peax,peax
jz endStartService

mov pesi,peax ;(R/E)SI => result of OpenSCManagerA

;open handle to service TV's server v3
argpush 3,0x0010           ;SERVICE_START
argpush 2,service_name     ;name of the service to open
argpush 1,peax             ;hServiceManager
call [OpenServiceA]
test peax,peax
jz OpenServiceFailed3

mov pebx,peax;(R/E)BX => handle to service

argpush 3,0
argpush 2,0
argpush 1,pebx
call [StartService]

mov edi,eax ;save result of StartService

;close handle to service
argpush 1,pebx
call [CloseServiceHandle]

mov eax,edi ;restore result of StartService

OpenServiceFailed3:
mov edi,eax ;save return value

;close handle to service manager
argpush 1,pesi
call [CloseServiceHandle]

mov eax,edi ;restore return value

endStartService:
call dleave
ret

;------------------------------------------------------------------------------
;This function will return a boolean if the process is running as system
;input:
;   none
;output:
;    (R/E)AX=> boolean, true if running as system and false if not.
;------------------------------------------------------------------------------
IsRunningInteractive:
call denter

;Windows NT does not support function CheckTokenMembership, so see if it exist in x86
if used is32
  ;Try to  receive pointer from function 'CheckTokenMembership', since it does not exist in Windows NT
  push advapi32_name
  call loadmodule

  call pushFunctionCheckTokenMembership
  db 'CheckTokenMembership',0
  pushFunctionCheckTokenMembership:
  push peax
  call getfunc

  mov pedi,peax ;(R/E)DI => pointer to CheckTokenMembership
end if

sub pesp,6+2+psize*2 ;size in order: SID_IDENTIFIER_AUTHORITY, fixup and PSID

mov dword [pesp+psize+2],0
mov byte [pesp+psize+2+4],0

mov byte [pesp+2+psize+5],5 ;SID_IDENTIFIER_AUTHORITY = SECURITY_NT_AUTHORITY
mov pebx,pesp ;(R/E)BX => pointer to PSID
lea pesi,pdword [pesp+psize+2] ;(R/E)SI => pointer to SID_IDENTIFIER_AUTHORITY

MakeStack

argpush 11,pebx ;PSID
argpush 10,0    ;dwSubAuthority7
argpush 9,0     ;dwSubAuthority6
argpush 8,0     ;dwSubAuthority5
argpush 7,0     ;dwSubAuthority4
argpush 6,0     ;dwSubAuthority3
argpush 5,0     ;dwSubAuthority2
argpush 4,0     ;dwSubAuthority1   (use 0x220 for admin instead of system)
argpush 3,0x4   ;dwSubAuthority0 (= SECURITY_INTERACTIVE_RID) (user 0x12 SECURITY_LOCAL_SYSTEM_RID for accunt SYSTEM) (use 0x20 for admin instead of system)
argpush 2,1     ;nSubAuthorityCount (use 2 for admin instead of system)
argpush 1,pesi  ;pointer to SID_IDENTIFIER_AUTHORITY
call [AllocateAndInitializeSid]
test eax,eax
jz endIsRunningInteractive

if used is32
test pedi,pedi ;test if function CheckTokenMembership exist
jnz NotWindowsNT

  call [GetCurrentProcess]

  push pesi  ;[ESI] => TokenHandle
  push 8    ;DesiredAccess = TOKEN_QUERY
  push eax  ;ProcessHandle
  call [OpenProcessToken]
  test eax,eax
  jz endIsRunningInteractive

  push 0 ;ReturnLength

  ;receive the length of the TokenInformation buffer size
  push esp         ;pointer to ReturnLength
  push 0           ;TokenInformationLength
  push 0           ;TokenInformation
  push 2           ;TokenInformationClass = TokenGroups
  push dword [pesi] ;TokenHandle
  call [GetTokenInformation]
  pop edi ;requierd length of TokenInformation
  test edi,edi
  jz endIsRunningInteractiveAtNTSystemErrorGetTokenInformation

  ;create buffer to receive the TokenInformation
  push edi
  call GetMemLocation

  push 0 ;ReturnLength

  ;receive TokenInformation of this process
  push esp         ;pointer to ReturnLength
  push edi         ;TokenInformationLength
  push eax         ;TokenInformation
  push 2           ;TokenInformationClass = TokenGroups
  push dword [pesi] ;TokenHandle

  mov edi,eax ;EDI => pointer to TokenInformation

  call [GetTokenInformation]

  add pesp,0x4 ;remove ReturnLength from stack

  test eax,eax
  jz endIsRunningInteractiveAtNTSystem

  push esi ;save process token created by OpenProcessToken

  mov esi,edi         ;ESI => pointer to TokenInformation
  lodsd               ;EAX => GroupCount
  mov ecx,eax         ;ECX => GroupCount
  ;mov eax,dword [pebx] ;Sid we want to find

  ;Loop all tokens and see if one match the systems token
  loopTokens:
    lodsd ;EAX => pointer to SID

    push ecx ;save counter

    ;check if we found the interactive SID in this token
    push dword [pebx]
    push eax
    call [EqualSid]

    pop ecx ;restore counter

    test eax,eax
    jz nextToken

      ;check acces flag
      lodsd ;get Attributes
      test eax,0x4
      jz loopTokens ;the token is not enabled

      ;found token
      xor eax,eax
      inc eax
      jmp SidFound

  nextToken:
    lodsd ;skip Attributes
  loop loopTokens

  mov eax,ecx ;EAX = 0

  SidFound:
  pop esi ;restoreprocess token created by OpenProcessToken on EDI

  endIsRunningInteractiveAtNTSystem:
  push eax ;save return value

  ;remove the buffer created to receive the TokenInformation
  push edi
  call FreeMemLocation

  pop eax ;restore return value

  endIsRunningInteractiveAtNTSystemErrorGetTokenInformation:
  push eax ;save return value

  ;close process token created by OpenProcessToken
  push dword [pesi]
  call [CloseHandle]

  pop eax ;restore return value
  jmp endIsRunningInteractive

NotWindowsNT:
end if

argpush 3,pesi           ;IsMember
argpush 2,pdword [pebx]  ;SidToCheck
argpush 1,0              ;TokenHandle
if used is32
  call pedi
else
  call [CheckTokenMembership]
end if
test eax,eax
jz endIsRunningInteractive

mov eax,dword [pesi] ;return if the process is a member of the system group.

endIsRunningInteractive:
push peax

argpush 1,pdword [pebx]
call [FreeSid]

pop peax

call dleave
ret

;------------------------------------------------------------------------------
;This function will check if the executable should behave as service, and if so, start the service dispatcher
;input:
;   none
;output:
;    (R/E)AX=> boolean, true if the executable is running as service and false if not.
;------------------------------------------------------------------------------
CheckAndStartService:
call denter
MakeStack

if used is32
  test [is9x],1
  jz isNTSystem

    ;Try to  receive pointer from function 'ChangeServiceConfig2A', since it does not exist in Windows NT
    call pushKernel
    db 'kernel32.dll',0
    pushKernel:
    call loadmodule

    call pushRegisterProcessAsService
    db 'RegisterServiceProcess',0
    pushRegisterProcessAsService:
    push peax
    call getfunc
    mov edi,eax

    call [GetCurrentProcessId]

    push 1   ;RSP_SIMPLE_SERVICE
    push eax ;processId
    call edi ;register process as service [RegisterServiceProcess]

    xor eax,eax ;return zero that indicates that the server services haven't been started yet!
    jmp endCheckAndStartService

  isNTSystem:
end if

;if this process is started as user INTERACTIVE this a user process, if not this is a service process!
call IsRunningInteractive
test eax,eax
jz isSystemUser

;we run as user, tro to start the service "TV's server" from mthe Service Manager
call startService
jmp endCheckAndStartService

isSystemUser:
;Start the service dispatcher
argpush 1,ServiceTable
call [StartServiceCtrlDispatcher]

endCheckAndStartService:
call dleave
ret

;------------------------------------------------------------------------------
;This function will report to the system's question to initiate the service
;input:
;   none
;output:
;    (R/E)AX=> boolean, true if the executable is running as service and false if not.
;------------------------------------------------------------------------------
service_main:
StartProc 2 ;set param sess on stack
call denter

if used New_Memory_management & used MemoryMultiThreaded
  call initNewPointerListInTEBForService
end if

;if used New_Memory_management & used MemoryMultiThreaded
;call initNewPointerListInTEBForService
;end if

push 100
call GetMemLocation

;create room on stack for the filename of this application  and zero this string
sub pesp,288
mov pebx,pesp

push 0
push 0
mov pecx,pesp

MakeStack

;register the function that handles service-messages
argpush 2,ServiceProc
argpush 1,pecx
call [RegisterServiceCtrlHandler]
mov [Handle],peax


;set status to start pending
push 0x2 ;SERVICE_START_PENDING
call SetServiceState

;call GetModuleFileName to receive the path to thie application
argpush 3,260
argpush 2,pebx
argpush 1,0
call [GetModuleFileNameA]

;add a end the of filename and add a quote there
lea pedi,pdword [pebx+260]

xor ecx,ecx  ; zero ECX
dec pecx     ; (R/E)CX => -1  (scasb will be repeated for 0xFFFFFFFF(FFFFFFFF) times, or until zero byte is found)
mov eax,'\'  ; byte to find with scasb => '\\'

STD ;execute scasb function backward
repnz scasb   ;loop string until something else then a zero byte is found
CLD ;clear backward direction

;place a zero byte at the slash to get the executable direcotry
mov byte [pedi+2],0

;set current direcotry to the directory the executable is at
argpush 1,pebx
call [SetCurrentDirectoryA]

call StartTVServer

;set status to start running
push 0x4 ;SERVICE_RUNNING
call SetServiceState

xor eax,eax
inc eax

call dleave
ret FunctionReturnFactor*2

;------------------------------------------------------------------------------
;This function will handle service messages
;input:
;   1:fdwControl
;output:
;    none
;
;fdwControl:
;       0x1 SERVICE_CONTROL_STOP
;       0x2 SERVICE_CONTROL_PAUSE
;       0x3 SERVICE_CONTROL_CONTINUE
;       0x4 SERVICE_CONTROL_INTERROGATE
;       0x5 SERVICE_CONTROL_SHUTDOWN
;       0x6 SERVICE_CONTROL_PARAMCHANGE
;       0x7 SERVICE_CONTROL_NETBINDADD
;       0x8 SERVICE_CONTROL_NETBINDREMOVE
;       0x9 SERVICE_CONTROL_NETBINDENABLE
;       0xA SERVICE_CONTROL_NETBINDDISABLE
;------------------------------------------------------------------------------
ServiceProc:
StartProc 1
call denter

cmp dword [pebp+psize*2],0x1 ;SERVICE_CONTROL_STOP
;je StopService
jne NoReturn

;---StopService
;StopService:
;-push 0x3
;-call SetServiceState

;-push 0
;-push [pId]
;-call [TerminateProcess]

push 0x3 ;SERVICE_STOP_PENDING
call SetServiceState

call ShutdownServer_internal

waitForListenSessionsToClose:
  push [listenSessions]
  call CountItems
  test eax,eax
  jz endWaitForListenSessionsToClose

  push 1000
  call sleep
jmp waitForListenSessionsToClose

endWaitForListenSessionsToClose:

push 0x1 ;SERVICE_STOPPED
call SetServiceState

NoReturn:
mov eax,1

call dleave
ret FunctionReturnFactor