
;TV's server version 3.10
;Copyright (c) 2014 - 2019, Tycho Veltmeijer.
;All rights reserved.

FD_READ equ 0x1
FD_WRITE equ 0x2
FD_CLOSE equ 0x20

;------------------------------------------------------------------------------
;This function will initilize the SOCKET stack
;return: 1 if NO error, else zero
;-----------------------------------------------------------------------------
initilize_socket_api:
call denter
MakeStack

sub pesp, 416+(psize*2)
lea peax,pdword [pesp+psize*2]
argpush 2,peax
argpush 1,0x202
Call [WSAStartup]
;Return value of WSAStartup is zero when NO ERROR

xor ecx,ecx
xor edx,edx
inc edx

;if EAX>0 then EAX = ECX (0)
;if EAX==0 then EAX = EDX (1)
test eax,eax
cmovz eax,edx
cmovnz eax,ecx

call dleave
ret

;------------------------------------------------------------------------------
;This function will initilize a listen SOCKET
;default: IP v4 on TCP
;param:
;  flags: 0x1 Use IP v6
;         0x2 Use UDP
;  port:  The port to bind the socket to
;-----------------------------------------------------------------------------
CreateSocket:
call denter
MakeStack

;set the address family
mov ebx,2  ;AF_INET
mov ecx,23 ;AF_INET6

test dword [pebp+psize*2] ,0x1 ;test the flag IPv6
cmovnz ebx,ecx ;set EBX on AF_INET6

;set the type of socket and protocol
mov edi,1 ;SOCK_STREAM
mov ecx,2 ;SOCK_DGRAM

mov esi,6 ;IPPROTO_TCP
mov edx,17; IPPROTO_UDP

test dword [pebp+psize*2] ,0x2 ;test the flag UDP
cmovnz edi,ecx ;set edi on SOCK_DGRAM
cmovnz esi,edx ;set esi on IPPROTO_UDP

;Create the socket
argpush 3,esi ;IPPROTO_TCP or IPPROTO_UDP
argpush 2,edi ;SOCK_STREAM or SOCK_DGRAM
argpush 1,ebx ;AF_INET or AF_INET6
call [socket]
;check return value on INVALID_SOCKET
inc peax
test peax,peax
jz endCreateSocket
dec peax

mov pedx,peax

;size parameter
mov pesi,0x10

;make room on stack for sockaddr_in structure
sub pesp,0x10

test dword [pebp+psize*2] ,0x1 ;test the flag IPv6
jz createIPv4Structure

;add extra size on stack to create sockaddr_in6 structure
sub pesp,0x10
add pesi,0x10

createIPv4Structure:
mov pedi,pesp ;pointer to sockaddr_in(6)

;Zero the structure
xor eax,eax
mov ecx,esi ;size of the structure
rep stosb

;pointer to sockaddr_in(6)
mov pedi,pesp ;pointer to sockaddr_in(6)

;fill in member family
mov word [pedi],bx ;set family address


;Fill in member port
mov eax,dword [pebp+psize*3] ;get Port from stack
xchg al, ah ;convert to big-endian
mov word [pedi+2],ax ;fill in the port number

mov pebx,pedx ;save socket on EBX

MakeStack ;for x64 support, create room for the bind and listen parameters

argpush 3,pesi ;Size of sockaddr_in structure
argpush 2,pedi ;Pointer to sockaddr_in structure
argpush 1,pebx ;socket
call [bind]

inc eax
test eax,eax
jz errCreateSocket

test dword [pebp+psize*2] ,0x2 ;test the flag UDP
jnz dontListen

argpush 2,0x7FFFFfff ;backlog
argpush 1,pebx     ;socket
call [listen]

inc eax
test eax,eax
jz errCreateSocket

dontListen:
mov peax,pebx

endCreateSocket:
call dleave
ret psize*2

;-
errCreateSocket:
argpush 1,pebx
call [closesocket]

xor eax,eax
jmp endCreateSocket

;------------------------------------------------------------------------------
;This function will wait for a client to connect
;param:
;  socket: the socket to listen to
;  sockaddr_in: pointer to sockaddr_in strucutre for remote address
;  addrlen: size of sockaddr_in
;return: -1 or a socket descriptor
;-----------------------------------------------------------------------------
Accept:
call denter

;if used is64
;  push 0 ;align stack
;end if
push pdword [pebp+psize*4]
mov pedi,pesp
MakeStack

recallAccept:
mov peax,pdword [pebp+psize*4]
mov pdword [pedi],peax

argpush 3,pedi                  ;addrlen
argpush 2,pdword [pebp+psize*3] ;sockaddr_in
argpush 1,pdword [pebp+psize*2] ;Socket
call [accept]


;if used is64
;  add rsp,psize*2
;else
;  add pesp,psize
;end if

mov ecx,eax
inc ecx
test ecx,ecx
jnz endAccept

;WSAGetLastError is a redirect to GetLastError(), Sleep() will set this to "No Error"
call [WSAGetLastError]
mov ebx,eax

argpush 1,1
call [Sleep]

cmp ebx,10054 ;WSAECONNRESET
je recallAccept

cmp ebx,10004 ;WSAEINTR
je recallAccept

cmp ebx,10024 ;WSAEMFILE
je recallAccept

xor eax,eax
dec eax

endAccept:
call dleave
ret psize*3

;------------------------------------------------------------------------------
;This function will return the local address on sockaddr_in structure
;param:
;  socket: the socket to listen to
;  sockaddr_in: pointer to sockaddr_in strucutre for local address
;  addrlen: size of sockaddr_in
;return: socket
;-----------------------------------------------------------------------------
GetLocalAddress:
call denter

push pdword [pebp+psize*4]
mov pecx,pesp

MakeStack

mov pebx,pdword [pebp+psize*2] ;socket

argpush 3,pecx
argpush 2,pdword [pebp+psize*3]
argpush 1,pebx
call [getsockname]

mov peax,pebx ;return socket
call dleave
ret psize*3

;------------------------------------------------------------------------------
;This function will create a event for write and read events for a socket
;param:
;  event: the event
;return: True or false
;-----------------------------------------------------------------------------
if used ResetSocketEvent
ResetSocketEvent:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]            ;event
call [WSAResetEvent]

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function willreset event like it's not triggert
;param:
;  socket: the socket to create the event on
;return: -1 or a event descriptor
;-----------------------------------------------------------------------------
CreateSocketEvent:
call denter
MakeStack

call [WSACreateEvent]
mov pebx,peax

argpush 3,FD_READ or FD_WRITE              ;trigger
argpush 2,peax                             ;event
argpush 1,pdword [pebp+psize*2]            ;SOCKET
call [WSAEventSelect]

mov peax,pebx
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will create a event for close events for a socket
;param:
;  socket: the socket to create the event on
;return: -1 or a event descriptor
;-----------------------------------------------------------------------------
CreateSocketEventClose:
call denter
MakeStack

call [WSACreateEvent]
mov pebx,peax

argpush 3,FD_CLOSE                         ;trigger
argpush 2,peax                             ;event
argpush 1,pdword [pebp+psize*2]            ;SOCKET
call [WSAEventSelect]

mov peax,pebx
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will check if WSA event was signaled
;param:
;  event: the event to be
;return: True or false
;-----------------------------------------------------------------------------
IsWSAEventSignaled:
call denter
MakeStack

argpush 5,0      ;fAlertable
argpush 4,0      ;dwTimeout
argpush 3,1      ;fWaitAll
if used is32
  lea eax,pdword [pebp+psize*2]
  push peax      ;lphEvents
else 
  lea rdx,pdword [pebp+psize*2]  ;lphEvents
end if
argpush 1,1      ;cEvents
call [WSAWaitForMultipleEvents]
test eax,eax ;WSA_WAIT_EVENT_0
jnz NotSignaled

inc eax ;TRUE

endIsWSAEventSignaled:
call dleave
ret psize

NotSignaled:
xor eax,eax
jmp endIsWSAEventSignaled

;------------------------------------------------------------------------------
;This function will read the port from sockaddr_in and convert it to litle-endian
;param:
;  sockaddr_in
;return: port of sockaddr_in
;-----------------------------------------------------------------------------
GetPort:
mov peax,pdword [pesp+psize] ;sockaddr_in
mov ax,word [peax+2] ;sin_port
xchg al, ah ;convert to litle-endian

;clear higher word of eax
and eax,0xFFFF
ret psize

;------------------------------------------------------------------------------
;This function will return the size of and the pointer to sin6_addr or sin_addr
;param:
;  pointer to {sizeof(sockaddr_in), sockaddr_in}
;return: (R/E)AX => pointer to sin6_addr or sin_addr
;        (R/E)CX => size of sin6_addr or sin_addr
;-----------------------------------------------------------------------------
GetSinAddr:
mov peax,pdword [pesp+psize] ;pointer to {sizeof(sockaddr_in), sockaddr_in}
mov pecx,pdword [peax]       ;sizeof(sockaddr_in)
add peax,psize               ;skip to pointer sockaddr_in or sockaddr6_in
shr ecx,2                    ;ECX = ECX / 4
add peax,pecx                ;seek to the sin_addr or sin6_addr structure
cmp ecx,4                    ;if it's IPv4, the shr instruction was enough to calculate the size of sin_addr
je endGetSinAddr

shl ecx,1                    ;calculate the size of sin6_addr

endGetSinAddr:
ret psize

;------------------------------------------------------------------------------
;This function will convert the address to a readable string
;param:
;  pointer to {sizeof(sockaddr_in), sockaddr_in}
;  output pointer
;  pointer to size of output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
ConvertAddress:
call denter

mov pedx,pdword [pebp+psize*5] ;flags
mov pesi,pdword [pebp+psize*4] ;size pointer

push pdword [pebp+psize*2]
call GetSinAddr
;(R/E)AX => pointer to sin6_addr or sin_addr
;(R/E)CX => size of sin6_addr or sin_addr

test pedx,CSI_ASCII_STRING ;test on flag Ascii string
jnz isAscii

shr dword [pesi],1 ;length devide by 2 (since 1 wchar = 2 bytes)

isAscii:
cmp ecx,4
je isIPv4     ;is sin_addr

mov pebx,IPv6AddressToString
mov ecx,40 ;ECX = 40
jmp ipVariablesSet


isIPv4:
mov pebx,IPv4AddressToString
shl ecx,2 ;ECX = 16 (4*4)
mov eax,dword [peax] ;get sib_addr from pointer

ipVariablesSet:
cmp dword [pesi],ecx
jae bufferIsSufficient

  mov dword [pesi],ecx  ;set requested buffer size
  xor eax,eax           ;clear return value
  jmp endConvertAddress

bufferIsSufficient:
push pedx                       ;flags
push pdword [pebp+psize*3]      ;output pointer
push peax                       ;sin_addr
call pebx                       ;call IPv6AddressToString or IPv4AddressToString

xor eax,eax
inc eax ;function succes

endConvertAddress:
call dleave
ret psize*4


;------------------------------------------------------------------------------
;This function will convert the IPv4 address to a readable string
;param:
;  sin_addr
;  output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
IPv4AddressToString:
call denter

mov eax,dword [pebp+psize*2]   ;sin_addr
mov ecx,4 ;4 bytes to process
xor edx,edx                    ;zero edx
mov pedi,pdword [pebp+psize*3] ;output pointer
mov bl,10                      ;divider
mov esi,dword [pebp+psize*4]   ;flags
cld
jmp processGroups

processGroupsWithDot:
  mov byte [pedi],'.' ;put a dot between every byte
  inc pedi

  test esi,CSI_ASCII_STRING
  jnz isOutputAscii3

  mov byte [pedi],0
  inc pedi

  isOutputAscii3:

processGroups:
  mov bh,ah ;save AH

  nextDigit:
    xor ah,ah    ;zero ah (=rest)
    div bl       ;devide by 10

    add ah,30h   ;add 0x30 (='0')

    push peax    ;push number on stack
    inc edx      ;increment value with pushed numbers on stack

    test al,al   ;if byte = zero, we are done
  jnz nextDigit

  ;set every calculated number on the string
  setChars:
    pop peax

    mov byte [pedi],ah
    inc pedi

    test esi,CSI_ASCII_STRING
    jnz isOutputAscii

    mov byte [pedi],0
    inc pedi

    isOutputAscii:

    dec edx
  jnz setChars

  mov ah,bh ;restore ah
  shr eax,8
loop processGroupsWithDot

xor ax,ax

test esi,CSI_ASCII_STRING
jnz isOutputAscii2
stosw
jmp endIPv4AddressToString

isOutputAscii2:
stosb

endIPv4AddressToString:
call dleave
ret (psize*3)

;------------------------------------------------------------------------------
;This function will convert the IPv6 address to a readable string
;param:
;  pointer to sin6_addr
;  output pointer
;  flags 0x0     Unicode string
;        0x10000 Ascii string

;return: -
;-----------------------------------------------------------------------------
IPv6AddressToString:
call denter

mov pesi,pdword [pebp+psize*2]   ;(R/E)SI => sin_addr
mov ecx,8 ;8 words to process
mov pedi,pdword [pebp+psize*3] ;output pointer
mov ebx,dword [pebp+psize*4]   ;flags in higher word of ebx
mov bx,0x10                    ;divider  in lower word of ebx
cld
jmp processGroupsv6

processGroupsv6WithDubbleDot:
  mov byte [pedi],':' ;put a dot between every byte
  inc pedi

  test ebx,CSI_ASCII_STRING
  jnz isOutputv6Ascii3

  xor al,al
  stosb

  isOutputv6Ascii3:

processGroupsv6:
  lodsw ;load word [esi] on ax
  xchg al, ah ;convert to litle-endian
  push pecx ;save ecx
  xor ecx,ecx ;clear ecx

  nextDigitv6:
    xor edx,edx  ;zero (e)dx
    div bx       ;devide by 0x10

    add dx,30h   ;add 0x30 (='0')
    cmp dx,'9'
    jna notAlphabetical

    add dx,7

  notAlphabetical:
    push pedx    ;push number on stack
    inc ecx      ;increment value with pushed numbers on stack

    test al,al   ;if byte = zero, we are done
  jnz nextDigitv6

  ;set every calculated number on the string
  setCharsv6:
    pop peax
    stosb

    test ebx,CSI_ASCII_STRING
    jnz isOutputv6Ascii2

    xor al,al
    stosb

    isOutputv6Ascii2:
  loop setCharsv6

  pop pecx ;restore ecx
loop processGroupsv6WithDubbleDot

xor al,al
stosb

test ebx,CSI_ASCII_STRING
jnz isOutputv6Ascii

stosb

isOutputv6Ascii:

call dleave
ret (psize*3)

;------------------------------------------------------------------------------
;This function will close an event
;param:
;  event: event descriptor
;return: boolean
;-----------------------------------------------------------------------------
CloseSocketEvent:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]            ;event
call [WSACloseEvent]

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will close a socket, no shutdown or annything
;param:
;  Socket
;-----------------------------------------------------------------------------
CloseSocket:
CloseGeneralSocket:
call denter
MakeStack

mov pedi,pdword [pebp+psize*2]
jmp _closeSocketUnsafe

;------------------------------------------------------------------------------
;This function will send and receive remaining data and finaly close the socket
;param:
;  Socket
;-----------------------------------------------------------------------------
SaflyCloseTCPSocket:
call denter
MakeStack

;-call [WSACreateEvent]
;-mov pebx,peax

mov pedi,pdword [pebp+psize*2] ;SOCKET

;-argpush 3,32;(=FD_CLOSE)        ;event
;-argpush 2,pebx                  ;Event
;-argpush 1,pedi                  ;SOCKET
;-call [WSAEventSelect]
;-inc eax
;-test eax,eax
;-jz saflyCloseSocket

argpush 2,1;(=SD_SEND)          ;SD_SEND
argpush 1,pedi                  ;SOCKET
call [shutdown]

sub pesp,80
if used is64
   mov r12,pesp
end if

MakeStack

recvRemainingData:
  if used is32
     mov pecx,pesp
  end if

  argpush 4,0                     ;flags
  argpush 3,80                    ;length of buffer
  if used is32
    push ecx
  else
    argpush 2,r12                 ;pointer to the buffer
  end if
  argpush 1,pedi                  ;SOCKET
  call [recv]

test eax,eax
jz saflyCloseSocket

inc eax
test eax,eax
jnz recvRemainingData

saflyCloseSocket:

;-argpush 1,pebx
;-call [WSACloseEvent]

_closeSocketUnsafe:
argpush 1,pedi
call [closesocket]

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will send data
;param:
;  1:Socket
;  2:flags
;  3:timeout
;  4:buf
;  5:&len
;
;flags: RSF_RETURN_ON_ERROR (0x1) Return on error, do not try to resolve the error
;
;Return: CE_NOERR (0x0)
;        CE_NO_BUFFER_AVAILABLE (0x1)
;        CE_CONNECTION_CLOSED (0x2)
;        CE_UNKNOWN_ERROR (0x3)
;Return on &len: The data that is send (on success)
;-----------------------------------------------------------------------------
SendSocketData:
call denter
MakeStack

;(R/E)BX = SOCKET
mov pebx,pdword [pebp+psize*2]
;(R/E)SI = buf
mov pesi,pdword [pebp+psize*5]
;(R/E)DI = &len
mov pedi,pdword [pebp+psize*6]

;skip the sleep command the first time
jmp sndSkipSleep

retrySendSocketData:
;Set thread in sleep state (so we do not eat CPU)
push 1
call sleep

sndSkipSleep:

if used is64            ;argpush 4,0
   xor r9d,r9d          ;flags
else
   push 0               ;flags
end if
argpush 3,dword [pedi]  ;len
argpush 2,pesi          ;Buff
argpush 1,pebx          ;SOCKET
call [send]
test eax,eax
jz sendRetIsZero

inc eax
test eax,eax
jnz successSendSocketData

mov ecx,CE_UNKNOWN_ERROR

;if RSF_RETURN_ON_ERROR is set we do not resolve any error
test dword [pebp+psize*3],RSF_RETURN_ON_ERROR
cmovnz eax,ecx ;set error code and return if the flag is set
jnz endSendSocketData

;Try to resolve the error

sendRetIsZero:
;Get the last Socket error
;WSAGetLastError is a redirect to GetLastError(), Sleep() will set this to "No Error"
call [WSAGetLastError]

cmp eax,10055 ;WSAENOBUFS
je retrySendSocketData

;cmp eax,10040 ;WSAEMSGSIZE
;je retrySendData

mov ecx,CE_NO_BUFFER_AVAILABLE
cmp eax,10014 ;WSAEFAULT
cmovz eax,ecx
je endSendSocketData

cmp eax,10036 ;WSAEINPROGRESS    block
je timeBlockedConnection ;retrySendSocketData

cmp eax,10035;WSAEWOULDBLOCK      block
je timeBlockedConnection ;retrySendSocketData

cmp eax,10065;WSAEHOSTUNREACH      block
je timeBlockedConnection ;retrySendSocketData

cmp eax,10004;WSAEINTR         block
je timeBlockedConnection ;retrySendSocketData

mov eax,CE_CONNECTION_CLOSED
jmp endSendSocketData

jmp retrySendSocketData

timeBlockedConnection:
;wait 30 ms for the connectio to unblock
push 30
call sleep

;test if RSF_USE_TIMEOUT is set
test dword [pebp+psize*3],RSF_USE_TIMEOUT
jz retrySendSocketData

;substract 100 ms from the maximum allowed timeout
sub dword [pebp+psize*4],30
jnc retrySendSocketData

mov eax,CE_TIMEOUT
jmp endSendSocketData

successSendSocketData:
dec eax
mov dword [pedi],eax

xor eax,eax ;no error

endSendSocketData:
call dleave
ret 5*psize

;------------------------------------------------------------------------------
;This function will receive data
;param:
;  1:Socket
;  2:flags
;  3:timeout
;  4:buf
;  5:&len
;
;flags: RSF_RETURN_ON_ERROR (0x1) Return on error, do not try to resolve the error
;       RSF_PEEK_DATA (0x2) Leave the data on the internal buffer
;       RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT (0x4) return afther one successful try receiving data from the client, do not check if 'size' is received
;
;Return: CE_NOERR (0x0)
;        CE_NO_BUFFER_AVAILABLE (0x1)
;        CE_CONNECTION_CLOSED (0x2)
;        CE_UNKNOWN_ERROR (0x3)

;Return on &len: The data that is received (on success)
;-----------------------------------------------------------------------------
RecvSocketData:
call denter
MakeStack

;(R/E)BX = SOCKET
mov pebx,pdword [pebp+psize*2]
;(R/E)DI = &len
mov pedi,pdword [pebp+psize*6]

; we cannot use MSG_WAITALL (0x100) since we need to keep track of the timeout value

;get flags
mov esi,dword [pebp+psize*3]

call [GetTickCount]
;r12 or ECX = start time of this function
;r13 or EDX = received len
if used is32
   mov ecx,eax
   xor edx,edx
else
   mov r12d,eax
   xor r13d,r13d
end if

;loopReceiveDataWithSaveTickCount:
if used is32
   push ecx      ;save tickcount
end if

xor eax,eax

mov ecx,esi
and ecx,RSF_PEEK_DATA    ;since RSF_PEEK_DATA == MSG_PEEK we can use it directly for recv

if used is32
   cmovnz edx,eax       ;zero len if RSF_PEEK_DATA is set
   push edx             ;save len value
else
   cmovnz r13d,eax      ;zero received buffer if RSF_PEEK_DATA is set
end if

;skip the sleep command the first time
jmp rcvSkipSleep

loopReceiveData:
;Set thread in sleep state (so we do not eat CPU)
push 1
call sleep

rcvSkipSleep:

mov ecx,esi
and ecx,RSF_PEEK_DATA    ;since RSF_PEEK_DATA == MSG_PEEK

mov eax,dword [pedi]    ;EAX = len

if used is32
   pop edx
   jnz peekFlagIsSet
     sub eax,edx          ;len - received buffer
   peekFlagIsSet:
   push edx
else
   jnz peekFlagIsSet
     sub eax,r13d         ;len - received buffer
   peekFlagIsSet:
end if

argpush 4,ecx                   ;flags
argpush 3,eax                   ;len
argpush 2,pdword [pebp+psize*5] ;buff
argpush 1,pebx                  ;SOCKET
call [recv]
test eax,eax
jz recvRetIsZero


;check on INVALID_SOCKET
inc eax ;incement EAX,  -1 will be 0
test eax,eax
;--cmovz eax,ecx
;--jz endRecvSocketData

;Since a Recv error cannot be resolved, return an error
jnz noErrorRecv

;An error accord, return if RSF_RETURN_ON_ERROR is set
mov ecx,CE_UNKNOWN_ERROR

test esi,RSF_RETURN_ON_ERROR
cmovnz eax,ecx
jnz endRecvSocketData

recvRetIsZero:
;Get the last Socket error
call [WSAGetLastError]

push peax

push 1
call sleep

pop peax

cmp eax,10035;WSAEWOULDBLOCK
je checkOnTimeout; loopReceiveData

mov ecx,CE_CONNECTION_CLOSED
cmp eax,10057 ;WSAENOTCONN
cmovz eax,ecx
jz endRecvSocketData


mov eax,CE_UNKNOWN_ERROR
jmp endRecvSocketData

noErrorRecv:
dec eax  ;restore EAX

test esi,RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT
jz dontReturnRecvSocketData

test eax,eax ;if no data received
jz checkOnTimeout

mov dword [pedi],eax  ;set &len
xor eax,eax ;CE_NO_ERROR
jmp endRecvSocketData

dontReturnRecvSocketData:
xor ecx,ecx  ;CE_NOERROR

;if PEEK flag is set, don't do ADD, do a move operation
test esi,RSF_PEEK_DATA

if used is32
   pop edx         ;pop received len
   cmovnz edx,eax  ;move data if PEEK flag is set (the next time it will give the same data, again and again and again)_
   jnz dontAddSizeSinceItIsPeek
   add edx,eax     ;add the received len from this try
   dontAddSizeSinceItIsPeek:
   push edx             ;save len value
   ;compair if the buffer is full
   cmp edx,dword [pedi]
else
   cmovnz r13d,eax  ;move data if PEEK flag is set (the next time it will give the same data, again and again and again)_
   jnz dontAddSizeSinceItIsPeek
   add r13d,eax    ;add the received len from this try
   dontAddSizeSinceItIsPeek:
   ;compair if the buffer is full
   cmp r13d,dword [pedi]
end if

cmovz eax,ecx  ;Set CE_NOERROR    ;cmovnc
jz endRecvSocketData              ;jae

test esi,RSF_PEEK_DATA    ;test on RSF_PEEK_DATA
jnz checkOnTimeout         ;jump if flag is set

;move the buffer +received bytes
add pdword [pebp+psize*5],peax

checkOnTimeout:
;check if we need to check on timeout
mov edx,dword [pebp+psize*4]  ;get timeout value
test edx,edx
jz loopReceiveData ;no Timout

call [GetTickCount]

if used is32
   sub eax,dword [pesp+0x4] ;restore start time of this function and substract it from current time
else
   sub eax,r12d
end if

cmp eax,dword [pebp+psize*4] ;check with TimeOut value
jb loopReceiveData ;not yet timed out

mov eax,CE_TIMEOUT ;err code

if used is32
   pop edx               ;pop received len
   mov dword [pedi],edx  ;set &len value
else
   mov dword [pedi],r13d ;set &len value
end if

endRecvSocketData:
call dleave
ret psize*5

;------------------------------------------------------------------------------
;This function will return the size of the send buffer for this socket
;param:
;  1:Socket
;
;Return: The size of the send buffer for this socket or zero

;-----------------------------------------------------------------------------
GetSendBufferSize:
push 0x1001 ;SO_SNDBUF
push pdword [pesp+psize*2]
call CallGetSockOpt
ret psize


;------------------------------------------------------------------------------
;This function will return the size of the receive buffer for this socket
;param:
;  1:Socket
;
;Return: The size of the send buffer for this socket or zero

;-----------------------------------------------------------------------------
GetRecvBufferSize:
push 0x1002 ;SO_RCVBUF
push pdword [pesp+psize*2] ;socket
call CallGetSockOpt
ret psize

;------------------------------------------------------------------------------
;This function will call getsockopt with intheger as result
;param:
;  1:Socket
;  2:optname
;
;Return: The size of the send buffer for this socket or zero

;-----------------------------------------------------------------------------
CallGetSockOpt:
call denter
MakeStack

if used is64
  lea r9,dword [rsp+psize*6] ;*optval
  lea rcx,dword [rsp+psize*7] ;*optlen

  mov dword [rcx],4 ;size of an integer

  argpush 5,rcx  ;*optlen
  argpush 4,r9   ;*optval
else
  push 0
  mov peax,pesp ;*optval
  push 4      ;size of an integer

  push esp    ;*optlen
  push eax    ;*optval
end if

argpush 3,pdword [pebp+psize*3]   ;optname
argpush 2,0xFFFF ;SOL_SOCKET      ;level
argpush 1,pdword [pebp+psize*2]   ;socket
call [getsockopt]
test eax,eax
jnz errCallGetSockOpt

if used is32
  pop eax ;pop size
  pop eax ;pop optval
else
  mov eax,dword [pesp+psize*6]
end if
jmp endCallGetSockOpt

errCallGetSockOpt:
xor eax,eax

endCallGetSockOpt:
call dleave
ret psize*2

;------------------------------------------------------------------------------
;This function will set the size of the send buffer for this socket
;param:
;  1:Socket
;  2:New send buffer size
;
;Return: Zero on succes

;-----------------------------------------------------------------------------
SetSendBufferSize:
push pdword [pesp+psize*2] ;New send buffer size
push 0x1001 ;SO_SNDBUF
push pdword [pesp+psize*3] ;socket
call CallSetSockOpt
ret psize*2

;------------------------------------------------------------------------------
;This function will set the size of the receive buffer for this socket
;param:
;  1:Socket
;  2:New send buffer size
;
;Return: Zero on succes

;-----------------------------------------------------------------------------
SetRecvBufferSize:
push pdword [pesp+psize*2] ;New receive buffer size
push 0x1002 ;SO_RCVBUF
push pdword [pesp+psize*3] ;socket
call CallSetSockOpt
ret psize*2

;------------------------------------------------------------------------------
;This function will call setsockopt for intheger options
;param:
;  1:Socket
;  2:optname
;  3:new value
;
;Return: Zero on succes

;-----------------------------------------------------------------------------
CallSetSockOpt:
call denter
MakeStack

if used is64
  lea r9,qword [rbp+psize*4]   ;optval (new value)

  argpush 5,4    ;*optlen
  argpush 4,r9   ;*optval
else
  lea eax,dword [ebp+psize*4]

  argpush 5,4    ;*optlen
  argpush 4,eax  ;*optval
end if

argpush 3,pdword [pebp+psize*3]   ;optname
argpush 2,0xFFFF ;SOL_SOCKET      ;level
argpush 1,pdword [pebp+psize*2]   ;socket
call [setsockopt]

endCallSetSockOpt:
call dleave
ret psize*3

;------------------------------------------------------------------------------
;This function will call setsockopt for intheger options
;param:
;  1:socket
;  2:onoff
;  3:keepalivetime
;  4:keepaliveinterval
;
;Return: boolean

;-----------------------------------------------------------------------------
SetKeepAlive:
call denter
MakeStack

if used is32
;--  sub pesp,0x18 ;Overlapped and lpcbBytesReturned
  ;empty _WSAOVERLAPPED
;--  mov dword [pesp],0
;--  mov dword [pesp+0x4],0
;--  mov dword [pesp+0x8],0
;--  mov dword [pesp+0xC],0
;--  mov dword [pesp+0x10],0
;--  mov pecx,pesp ;lpOverlapped
  push 0
  mov pedx,pesp
;--  lea edx,dword [pesp+0x14] ;lpcbBytesReturned
;--  mov dword [pedx],0 ;zero cbBytesReturned
else
  ;empty _WSAOVERLAPPED & cbBytesReturned
  mov qword [rsp+0x48],0
  mov qword [rsp+0x50],0
  mov qword [rsp+0x58],0

  ;move keepalivetime to apropiated place
  mov eax,dword [rbp+psize*4]
  mov dword [rbp+4+psize*3],eax
  ;move keepaliveinterval to apropiated place
  mov eax,dword [rbp+psize*5]
  mov dword [rbp+psize*4],eax

  lea rcx,qword [rsp+0x48] ;lpOverlapped
  mov rdx,rcx
  add rdx,0x18 ;lpcbBytesReturned
  mov dword [rdx],0 ;zero cbBytesReturned
end if

argpush 9,0;GetOverlappedResult_c     ;lpCompletionRoutine
argpush 8,0;pecx ;lpOverlapped
argpush 7,pedx ;lpcbBytesReturned
argpush 6,0    ;size output buffer
argpush 5,0    ;output buffer
argpush 4,0xC  ;cbInBuffer
if used is32
  lea eax,pdword [pebp+psize*3]
  push eax ;tcp_keepalive struct
else
  lea r8,pdword [pebp+psize*3] ;tcp_keepalive struct
end if
argpush 2,0x98000004 ;SIO_KEEPALIVE_VALS
argpush 1,pdword [pebp+psize*2]   ;socket
call [WSAIoctl]

inc eax ;-1 = 0 and 0 = 1

call dleave
ret psize*4

;--------------
;Callback function for OVERLAPPED functions
;   1:dwErrorCode
;   2:dwNumberOfBytesTransfered
;   3:lpOverlapped
;---------
;C implementation
;---------
;--if used is64
;--    GetOverlappedResult_c:
;--    StartProc 3

;--    mov rcx,retGetOverlappedResult_c
;--    xchg qword [rsp],rcx
;--    jmp GetOverlappedResult_asm
;--
;--    retGetOverlappedResult_c:
;--    sub rsp,3*psize
;--    EndProc 3
;--    jmp rcx
;--else
;--    GetOverlappedResult_c:
;--end if
;--GetOverlappedResult_asm:

;--mov peax,pdword [pebp+psize*4]
;--int3

